/* This file is generated from ppc64le.ops.  Do not edit directly. */

#include "config.h"
#include "dill.h"
#include "dill_internal.h"
#include "ppc64le.h"
#include <string.h>
static struct jmp_table_s ppc64le_jump_table_struct;
static arith_op3 ppc64le_arith_op3_table[dill_jmp_a3_size + 1];
static jmp_data  ppc64le_arith_op3_data[dill_jmp_a3_size + 1];
static arith_op3i ppc64le_arith_op3i_table[dill_jmp_a3_size + 1];
static jmp_data  ppc64le_arith_op3i_data[dill_jmp_a3_size + 1];
static arith_op2 ppc64le_arith_op2_table[dill_jmp_a2_size + 1];
static jmp_data  ppc64le_arith_op2_data[dill_jmp_a2_size + 1];
static branch_op ppc64le_branch_table[dill_jmp_branch_size + 1];
static branch_opi ppc64le_branchi_table[dill_jmp_branch_size + 1];
static jmp_data  ppc64le_branch_data[dill_jmp_branch_size + 1];
static compare_op ppc64le_compare_table[dill_jmp_compare_size + 1];
static compare_opi ppc64le_comparei_table[dill_jmp_compare_size + 1];
static jmp_data ppc64le_compare_data[dill_jmp_compare_size + 1];

jmp_table ppc64le_jump_table;
static void gen_ppc64le_jump_table()
{
    ppc64le_jump_table = &ppc64le_jump_table_struct;          
    memset(ppc64le_jump_table, 0, sizeof(struct jmp_table_s));
    /* arith3 */
    ppc64le_jump_table->jmp_a3 = ppc64le_arith_op3_table;
    memset(ppc64le_jump_table->jmp_a3, 0, sizeof(ppc64le_arith_op3_table));
    ppc64le_jump_table->a3_data = ppc64le_arith_op3_data;
    memset(ppc64le_jump_table->a3_data, 0, sizeof(ppc64le_arith_op3_data));
    /* arith3i */
    ppc64le_jump_table->jmp_a3i = ppc64le_arith_op3i_table;
    memset(ppc64le_jump_table->jmp_a3i, 0, sizeof(ppc64le_arith_op3i_table));
    ppc64le_jump_table->a3i_data = ppc64le_arith_op3i_data;
    memset(ppc64le_jump_table->a3i_data, 0, sizeof(ppc64le_arith_op3i_data));
    /* arith2 */
    ppc64le_jump_table->jmp_a2 = ppc64le_arith_op2_table;
    memset(ppc64le_jump_table->jmp_a2, 0, sizeof(ppc64le_arith_op2_table));
    ppc64le_jump_table->a2_data = ppc64le_arith_op2_data;
    memset(ppc64le_jump_table->a2_data, 0, sizeof(ppc64le_arith_op2_data));
    /* branch */
    ppc64le_jump_table->jmp_b = ppc64le_branch_table;
    memset(ppc64le_jump_table->jmp_b, 0, sizeof(ppc64le_branch_table));
    /* branchi */
    ppc64le_jump_table->jmp_bi = ppc64le_branchi_table;
    memset(ppc64le_jump_table->jmp_bi, 0, sizeof(ppc64le_branch_table));
    ppc64le_jump_table->b_data = ppc64le_branch_data;
    memset(ppc64le_jump_table->b_data, 0, sizeof(ppc64le_branch_data));
    /* compare */
    ppc64le_jump_table->jmp_c = ppc64le_compare_table;
    memset(ppc64le_jump_table->jmp_c, 0, sizeof(ppc64le_compare_table));
    /* comparei */
    ppc64le_jump_table->jmp_ci = ppc64le_comparei_table;
    memset(ppc64le_jump_table->jmp_ci, 0, sizeof(ppc64le_compare_table));
    ppc64le_jump_table->c_data = ppc64le_compare_data;
    memset(ppc64le_jump_table->c_data, 0, sizeof(ppc64le_compare_data));
    ppc64le_jump_table->jmp_a3[dill_jmp_addi] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_addi].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_addi].data2 = 266;
    ppc64le_jump_table->jmp_a3[dill_jmp_addu] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_addu].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_addu].data2 = 266;
    ppc64le_jump_table->jmp_a3[dill_jmp_addul] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_addul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_addul].data2 = 266;
    ppc64le_jump_table->jmp_a3[dill_jmp_addl] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_addl].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_addl].data2 = 266;
    ppc64le_jump_table->jmp_a3[dill_jmp_addp] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_addp].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_addp].data2 = 266;
    ppc64le_jump_table->jmp_a3[dill_jmp_subi] = ppc64le_swap_arith;
    ppc64le_jump_table->a3_data[dill_jmp_subi].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_subi].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_subu] = ppc64le_swap_arith;
    ppc64le_jump_table->a3_data[dill_jmp_subu].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_subu].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_subul] = ppc64le_swap_arith;
    ppc64le_jump_table->a3_data[dill_jmp_subul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_subul].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_subl] = ppc64le_swap_arith;
    ppc64le_jump_table->a3_data[dill_jmp_subl].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_subl].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_subp] = ppc64le_swap_arith;
    ppc64le_jump_table->a3_data[dill_jmp_subp].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_subp].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_mulu] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_mulu].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_mulu].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_mulul] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_mulul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_mulul].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_muli] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_muli].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_muli].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_mull] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_mull].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_mull].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_divu] = ppc64le_div;
    ppc64le_jump_table->a3_data[dill_jmp_divu].data1 = 459/*divwu*/;
    ppc64le_jump_table->a3_data[dill_jmp_divu].data2 = 0;
    ppc64le_jump_table->jmp_a3[dill_jmp_divul] = ppc64le_div;
    ppc64le_jump_table->a3_data[dill_jmp_divul].data1 = 457/*divdu*/;
    ppc64le_jump_table->a3_data[dill_jmp_divul].data2 = 1;
    ppc64le_jump_table->jmp_a3[dill_jmp_divi] = ppc64le_div;
    ppc64le_jump_table->a3_data[dill_jmp_divi].data1 = 491/*divw*/;
    ppc64le_jump_table->a3_data[dill_jmp_divi].data2 = 0;
    ppc64le_jump_table->jmp_a3[dill_jmp_divl] = ppc64le_div;
    ppc64le_jump_table->a3_data[dill_jmp_divl].data1 = 489/*divd*/;
    ppc64le_jump_table->a3_data[dill_jmp_divl].data2 = 1;
    ppc64le_jump_table->jmp_a3[dill_jmp_muli] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_muli].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_muli].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_mull] = ppc64le_XOFORM_arith;
    ppc64le_jump_table->a3_data[dill_jmp_mull].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_mull].data2 = 233;
    ppc64le_jump_table->jmp_a3[dill_jmp_modu] = ppc64le_mod;
    ppc64le_jump_table->a3_data[dill_jmp_modu].data1 = 0;
    ppc64le_jump_table->a3_data[dill_jmp_modu].data2 = 0;
    ppc64le_jump_table->jmp_a3[dill_jmp_modul] = ppc64le_mod;
    ppc64le_jump_table->a3_data[dill_jmp_modul].data1 = 0;
    ppc64le_jump_table->a3_data[dill_jmp_modul].data2 = 1;
    ppc64le_jump_table->jmp_a3[dill_jmp_modi] = ppc64le_mod;
    ppc64le_jump_table->a3_data[dill_jmp_modi].data1 = 1;
    ppc64le_jump_table->a3_data[dill_jmp_modi].data2 = 0;
    ppc64le_jump_table->jmp_a3[dill_jmp_modl] = ppc64le_mod;
    ppc64le_jump_table->a3_data[dill_jmp_modl].data1 = 1;
    ppc64le_jump_table->a3_data[dill_jmp_modl].data2 = 1;
    ppc64le_jump_table->jmp_a3[dill_jmp_andi] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_andi].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_andi].data2 = 28;
    ppc64le_jump_table->jmp_a3[dill_jmp_andu] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_andu].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_andu].data2 = 28;
    ppc64le_jump_table->jmp_a3[dill_jmp_andul] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_andul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_andul].data2 = 28;
    ppc64le_jump_table->jmp_a3[dill_jmp_andl] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_andl].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_andl].data2 = 28;
    ppc64le_jump_table->jmp_a3[dill_jmp_ori] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_ori].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_ori].data2 = 444;
    ppc64le_jump_table->jmp_a3[dill_jmp_oru] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_oru].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_oru].data2 = 444;
    ppc64le_jump_table->jmp_a3[dill_jmp_orul] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_orul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_orul].data2 = 444;
    ppc64le_jump_table->jmp_a3[dill_jmp_orl] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_orl].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_orl].data2 = 444;
    ppc64le_jump_table->jmp_a3[dill_jmp_xori] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_xori].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_xori].data2 = 316;
    ppc64le_jump_table->jmp_a3[dill_jmp_xoru] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_xoru].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_xoru].data2 = 316;
    ppc64le_jump_table->jmp_a3[dill_jmp_xorul] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_xorul].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_xorul].data2 = 316;
    ppc64le_jump_table->jmp_a3[dill_jmp_xorl] = ppc64le_log_arith;
    ppc64le_jump_table->a3_data[dill_jmp_xorl].data1 = 31;
    ppc64le_jump_table->a3_data[dill_jmp_xorl].data2 = 316;
    ppc64le_jump_table->jmp_a3[dill_jmp_lshi] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_lshi].data1 = 27;
    ppc64le_jump_table->a3_data[dill_jmp_lshi].data2 = DILL_I;
    ppc64le_jump_table->jmp_a3[dill_jmp_lshu] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_lshu].data1 = 27;
    ppc64le_jump_table->a3_data[dill_jmp_lshu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a3[dill_jmp_lshul] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_lshul].data1 = 27;
    ppc64le_jump_table->a3_data[dill_jmp_lshul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a3[dill_jmp_lshl] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_lshl].data1 = 27;
    ppc64le_jump_table->a3_data[dill_jmp_lshl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a3[dill_jmp_rshi] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_rshi].data1 = 792;
    ppc64le_jump_table->a3_data[dill_jmp_rshi].data2 = DILL_I;
    ppc64le_jump_table->jmp_a3[dill_jmp_rshl] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_rshl].data1 = 792;
    ppc64le_jump_table->a3_data[dill_jmp_rshl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a3[dill_jmp_rshu] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_rshu].data1 = 539;
    ppc64le_jump_table->a3_data[dill_jmp_rshu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a3[dill_jmp_rshul] = ppc64le_shift_arith;
    ppc64le_jump_table->a3_data[dill_jmp_rshul].data1 = 539;
    ppc64le_jump_table->a3_data[dill_jmp_rshul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a3[dill_jmp_addf] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_addf].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_addf].data2 = 0;
    ppc64le_jump_table->jmp_a3[dill_jmp_addd] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_addd].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_addd].data2 = 96;
    ppc64le_jump_table->jmp_a3[dill_jmp_subf] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_subf].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_subf].data2 = 8;
    ppc64le_jump_table->jmp_a3[dill_jmp_subd] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_subd].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_subd].data2 = 40;
    ppc64le_jump_table->jmp_a3[dill_jmp_mulf] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_mulf].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_mulf].data2 = 16;
    ppc64le_jump_table->jmp_a3[dill_jmp_muld] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_muld].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_muld].data2 = 48;
    ppc64le_jump_table->jmp_a3[dill_jmp_divf] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_divf].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_divf].data2 = 24;
    ppc64le_jump_table->jmp_a3[dill_jmp_divd] = ppc64le_farith;
    ppc64le_jump_table->a3_data[dill_jmp_divd].data1 = 60;
    ppc64le_jump_table->a3_data[dill_jmp_divd].data2 = 56;
    ppc64le_jump_table->jmp_a3i[dill_jmp_addi] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_addi].data1 = 14;
    ppc64le_jump_table->a3i_data[dill_jmp_addi].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_addu] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_addu].data1 = 14;
    ppc64le_jump_table->a3i_data[dill_jmp_addu].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_addul] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_addul].data1 = 14;
    ppc64le_jump_table->a3i_data[dill_jmp_addul].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_addl] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_addl].data1 = 14;
    ppc64le_jump_table->a3i_data[dill_jmp_addl].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_addp] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_addp].data1 = 14;
    ppc64le_jump_table->a3i_data[dill_jmp_addp].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_subi] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_subi].data1 = 8;
    ppc64le_jump_table->a3i_data[dill_jmp_subi].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_subu] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_subu].data1 = 8;
    ppc64le_jump_table->a3i_data[dill_jmp_subu].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_subul] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_subul].data1 = 8;
    ppc64le_jump_table->a3i_data[dill_jmp_subul].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_subl] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_subl].data1 = 8;
    ppc64le_jump_table->a3i_data[dill_jmp_subl].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_subp] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_subp].data1 = 8;
    ppc64le_jump_table->a3i_data[dill_jmp_subp].data2 = 266;
    ppc64le_jump_table->jmp_a3i[dill_jmp_mulu] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_mulu].data1 = 7;
    ppc64le_jump_table->a3i_data[dill_jmp_mulu].data2 = 233;
    ppc64le_jump_table->jmp_a3i[dill_jmp_mulul] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_mulul].data1 = 7;
    ppc64le_jump_table->a3i_data[dill_jmp_mulul].data2 = 233;
    ppc64le_jump_table->jmp_a3i[dill_jmp_muli] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_muli].data1 = 7;
    ppc64le_jump_table->a3i_data[dill_jmp_muli].data2 = 233;
    ppc64le_jump_table->jmp_a3i[dill_jmp_mull] = ppc64le_imm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_mull].data1 = 7;
    ppc64le_jump_table->a3i_data[dill_jmp_mull].data2 = 233;
    ppc64le_jump_table->jmp_a3i[dill_jmp_divu] = ppc64le_divi;
    ppc64le_jump_table->a3i_data[dill_jmp_divu].data1 = 459;
    ppc64le_jump_table->a3i_data[dill_jmp_divu].data2 = 0;
    ppc64le_jump_table->jmp_a3i[dill_jmp_divul] = ppc64le_divi;
    ppc64le_jump_table->a3i_data[dill_jmp_divul].data1 = 457;
    ppc64le_jump_table->a3i_data[dill_jmp_divul].data2 = 1;
    ppc64le_jump_table->jmp_a3i[dill_jmp_divi] = ppc64le_divi;
    ppc64le_jump_table->a3i_data[dill_jmp_divi].data1 = 491;
    ppc64le_jump_table->a3i_data[dill_jmp_divi].data2 = 0;
    ppc64le_jump_table->jmp_a3i[dill_jmp_divl] = ppc64le_divi;
    ppc64le_jump_table->a3i_data[dill_jmp_divl].data1 = 489;
    ppc64le_jump_table->a3i_data[dill_jmp_divl].data2 = 1;
    ppc64le_jump_table->jmp_a3i[dill_jmp_modu] = ppc64le_modi;
    ppc64le_jump_table->a3i_data[dill_jmp_modu].data1 = 0;
    ppc64le_jump_table->a3i_data[dill_jmp_modu].data2 = 0;
    ppc64le_jump_table->jmp_a3i[dill_jmp_modi] = ppc64le_modi;
    ppc64le_jump_table->a3i_data[dill_jmp_modi].data1 = 1;
    ppc64le_jump_table->a3i_data[dill_jmp_modi].data2 = 0;
    ppc64le_jump_table->jmp_a3i[dill_jmp_modul] = ppc64le_modi;
    ppc64le_jump_table->a3i_data[dill_jmp_modul].data1 = 0;
    ppc64le_jump_table->a3i_data[dill_jmp_modul].data2 = 1;
    ppc64le_jump_table->jmp_a3i[dill_jmp_modl] = ppc64le_modi;
    ppc64le_jump_table->a3i_data[dill_jmp_modl].data1 = 1;
    ppc64le_jump_table->a3i_data[dill_jmp_modl].data2 = 1;
    ppc64le_jump_table->jmp_a3i[dill_jmp_andi] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_andi].data1 = 28;
    ppc64le_jump_table->a3i_data[dill_jmp_andi].data2 = 28;
    ppc64le_jump_table->jmp_a3i[dill_jmp_andu] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_andu].data1 = 28;
    ppc64le_jump_table->a3i_data[dill_jmp_andu].data2 = 28;
    ppc64le_jump_table->jmp_a3i[dill_jmp_andul] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_andul].data1 = 28;
    ppc64le_jump_table->a3i_data[dill_jmp_andul].data2 = 28;
    ppc64le_jump_table->jmp_a3i[dill_jmp_andl] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_andl].data1 = 28;
    ppc64le_jump_table->a3i_data[dill_jmp_andl].data2 = 28;
    ppc64le_jump_table->jmp_a3i[dill_jmp_ori] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_ori].data1 = 24;
    ppc64le_jump_table->a3i_data[dill_jmp_ori].data2 = 444;
    ppc64le_jump_table->jmp_a3i[dill_jmp_oru] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_oru].data1 = 24;
    ppc64le_jump_table->a3i_data[dill_jmp_oru].data2 = 444;
    ppc64le_jump_table->jmp_a3i[dill_jmp_orul] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_orul].data1 = 24;
    ppc64le_jump_table->a3i_data[dill_jmp_orul].data2 = 444;
    ppc64le_jump_table->jmp_a3i[dill_jmp_orl] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_orl].data1 = 24;
    ppc64le_jump_table->a3i_data[dill_jmp_orl].data2 = 444;
    ppc64le_jump_table->jmp_a3i[dill_jmp_xori] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_xori].data1 = 26;
    ppc64le_jump_table->a3i_data[dill_jmp_xori].data2 = 316;
    ppc64le_jump_table->jmp_a3i[dill_jmp_xoru] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_xoru].data1 = 26;
    ppc64le_jump_table->a3i_data[dill_jmp_xoru].data2 = 316;
    ppc64le_jump_table->jmp_a3i[dill_jmp_xorul] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_xorul].data1 = 26;
    ppc64le_jump_table->a3i_data[dill_jmp_xorul].data2 = 316;
    ppc64le_jump_table->jmp_a3i[dill_jmp_xorl] = ppc64le_logimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_xorl].data1 = 26;
    ppc64le_jump_table->a3i_data[dill_jmp_xorl].data2 = 316;
    ppc64le_jump_table->jmp_a3i[dill_jmp_lshi] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_lshi].data1 = 21;
    ppc64le_jump_table->a3i_data[dill_jmp_lshi].data2 = DILL_I;
    ppc64le_jump_table->jmp_a3i[dill_jmp_lshu] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_lshu].data1 = 21;
    ppc64le_jump_table->a3i_data[dill_jmp_lshu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a3i[dill_jmp_lshul] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_lshul].data1 = 21;
    ppc64le_jump_table->a3i_data[dill_jmp_lshul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a3i[dill_jmp_lshl] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_lshl].data1 = 21;
    ppc64le_jump_table->a3i_data[dill_jmp_lshl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a3i[dill_jmp_rshi] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_rshi].data1 = 413;
    ppc64le_jump_table->a3i_data[dill_jmp_rshi].data2 = DILL_I;
    ppc64le_jump_table->jmp_a3i[dill_jmp_rshu] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_rshu].data1 = 413;
    ppc64le_jump_table->a3i_data[dill_jmp_rshu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a3i[dill_jmp_rshul] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_rshul].data1 = 413;
    ppc64le_jump_table->a3i_data[dill_jmp_rshul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a3i[dill_jmp_rshl] = ppc64le_shiftimm_arith;
    ppc64le_jump_table->a3i_data[dill_jmp_rshl].data1 = 413;
    ppc64le_jump_table->a3i_data[dill_jmp_rshl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a2[dill_jmp_comi] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_comi].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_comi].data2 = 124;
    ppc64le_jump_table->jmp_a2[dill_jmp_comu] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_comu].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_comu].data2 = 124;
    ppc64le_jump_table->jmp_a2[dill_jmp_comul] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_comul].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_comul].data2 = 124;
    ppc64le_jump_table->jmp_a2[dill_jmp_coml] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_coml].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_coml].data2 = 124;
    ppc64le_jump_table->jmp_a2[dill_jmp_negi] = (arith_op2)ppc64le_XOFORM_arith;
    ppc64le_jump_table->a2_data[dill_jmp_negi].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_negi].data2 = 104;
    ppc64le_jump_table->jmp_a2[dill_jmp_negu] = (arith_op2)ppc64le_XOFORM_arith;
    ppc64le_jump_table->a2_data[dill_jmp_negu].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_negu].data2 = 104;
    ppc64le_jump_table->jmp_a2[dill_jmp_negul] = (arith_op2)ppc64le_XOFORM_arith;
    ppc64le_jump_table->a2_data[dill_jmp_negul].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_negul].data2 = 104;
    ppc64le_jump_table->jmp_a2[dill_jmp_negl] = (arith_op2)ppc64le_XOFORM_arith;
    ppc64le_jump_table->a2_data[dill_jmp_negl].data1 = 31;
    ppc64le_jump_table->a2_data[dill_jmp_negl].data2 = 104;
    ppc64le_jump_table->jmp_a2[dill_jmp_noti] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_noti].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_noti].data2 = DILL_I;
    ppc64le_jump_table->jmp_a2[dill_jmp_notu] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_notu].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_notu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a2[dill_jmp_notul] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_notul].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_notul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a2[dill_jmp_notl] = (arith_op2)ppc64le_FORM2_arith;
    ppc64le_jump_table->a2_data[dill_jmp_notl].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_notl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a2[dill_jmp_negf] = ppc64le_XFORM2_farith;
    ppc64le_jump_table->a2_data[dill_jmp_negf].data1 = 40;
    ppc64le_jump_table->a2_data[dill_jmp_negf].data2 = 0;
    ppc64le_jump_table->jmp_a2[dill_jmp_negd] = ppc64le_XFORM2_farith;
    ppc64le_jump_table->a2_data[dill_jmp_negd].data1 = 40;
    ppc64le_jump_table->a2_data[dill_jmp_negd].data2 = 0;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswaps] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswaps].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswaps].data2 = DILL_S;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapus] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapus].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapus].data2 = DILL_US;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapi] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapi].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapi].data2 = DILL_I;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapu] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapu].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapu].data2 = DILL_U;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapul] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapul].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapl] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapl].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapl].data2 = DILL_L;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapf] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapf].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapf].data2 = DILL_F;
    ppc64le_jump_table->jmp_a2[dill_jmp_bswapd] = ppc64le_bswap;
    ppc64le_jump_table->a2_data[dill_jmp_bswapd].data1 = 0;
    ppc64le_jump_table->a2_data[dill_jmp_bswapd].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_beqc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqc].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqc].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bequc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bequc].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_bequc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_beqs] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqs].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqs].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bequs] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bequs].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_bequs].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_beqi] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqi].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqi].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bequ] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bequ].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_bequ].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bequl] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bequl].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_bequl].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_beql] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beql].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beql].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_beqp] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqp].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqp].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_beqd] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqd].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqd].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_beqf] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_beqf].data1 = dill_eq_code;
    ppc64le_jump_table->b_data[dill_jmp_beqf].data2 = DILL_F;
    ppc64le_jump_table->jmp_b[dill_jmp_bgec] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgec].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgec].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bgeuc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgeuc].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgeuc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_bges] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bges].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bges].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bgeus] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgeus].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgeus].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_bgei] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgei].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgei].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bgeu] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgeu].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgeu].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bgeul] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgeul].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgeul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_bgel] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgel].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgel].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_bgep] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgep].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgep].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_bged] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bged].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bged].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_bgef] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgef].data1 = dill_ge_code;
    ppc64le_jump_table->b_data[dill_jmp_bgef].data2 = DILL_F;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtc].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtc].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtuc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtuc].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtuc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_bgts] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgts].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgts].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtus] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtus].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtus].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_bgti] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgti].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgti].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtu] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtu].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtu].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtul] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtul].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtl] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtl].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtl].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtp] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtp].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtp].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtd] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtd].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtd].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_bgtf] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bgtf].data1 = dill_gt_code;
    ppc64le_jump_table->b_data[dill_jmp_bgtf].data2 = DILL_F;
    ppc64le_jump_table->jmp_b[dill_jmp_blec] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blec].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_blec].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bleuc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bleuc].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bleuc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_bles] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bles].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bles].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bleus] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bleus].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bleus].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_blei] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blei].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_blei].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bleu] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bleu].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bleu].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bleul] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bleul].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bleul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_blel] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blel].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_blel].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_blep] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blep].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_blep].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_bled] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bled].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_bled].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_blef] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blef].data1 = dill_le_code;
    ppc64le_jump_table->b_data[dill_jmp_blef].data2 = DILL_F;
    ppc64le_jump_table->jmp_b[dill_jmp_bltc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltc].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltc].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bltuc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltuc].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltuc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_blts] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blts].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_blts].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bltus] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltus].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltus].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_blti] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_blti].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_blti].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bltu] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltu].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltu].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bltul] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltul].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_bltl] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltl].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltl].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_bltp] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltp].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltp].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_bltd] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltd].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltd].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_bltf] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bltf].data1 = dill_lt_code;
    ppc64le_jump_table->b_data[dill_jmp_bltf].data2 = DILL_F;
    ppc64le_jump_table->jmp_b[dill_jmp_bnec] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnec].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnec].data2 = DILL_C;
    ppc64le_jump_table->jmp_b[dill_jmp_bneuc] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bneuc].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bneuc].data2 = DILL_UC;
    ppc64le_jump_table->jmp_b[dill_jmp_bnes] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnes].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnes].data2 = DILL_S;
    ppc64le_jump_table->jmp_b[dill_jmp_bneus] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bneus].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bneus].data2 = DILL_US;
    ppc64le_jump_table->jmp_b[dill_jmp_bnei] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnei].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnei].data2 = DILL_I;
    ppc64le_jump_table->jmp_b[dill_jmp_bneu] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bneu].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bneu].data2 = DILL_U;
    ppc64le_jump_table->jmp_b[dill_jmp_bneul] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bneul].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bneul].data2 = DILL_UL;
    ppc64le_jump_table->jmp_b[dill_jmp_bnel] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnel].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnel].data2 = DILL_L;
    ppc64le_jump_table->jmp_b[dill_jmp_bnep] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnep].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnep].data2 = DILL_P;
    ppc64le_jump_table->jmp_b[dill_jmp_bned] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bned].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bned].data2 = DILL_D;
    ppc64le_jump_table->jmp_b[dill_jmp_bnef] = ppc64le_branch;
    ppc64le_jump_table->b_data[dill_jmp_bnef].data1 = dill_ne_code;
    ppc64le_jump_table->b_data[dill_jmp_bnef].data2 = DILL_F;
    ppc64le_jump_table->jmp_bi[dill_jmp_beqc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bequc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_beqs] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bequs] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_beqi] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bequ] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bequl] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_beql] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_beqp] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgec] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgeuc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bges] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgeus] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgei] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgeu] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgeul] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgel] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgep] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtuc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgts] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtus] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgti] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtu] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtul] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtl] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bgtp] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blec] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bleuc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bles] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bleus] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blei] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bleu] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bleul] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blel] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blep] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltuc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blts] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltus] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_blti] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltu] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltul] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltl] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bltp] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bnec] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bneuc] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bnes] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bneus] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bnei] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bneu] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bneul] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bnel] = ppc64le_branchi;
    ppc64le_jump_table->jmp_bi[dill_jmp_bnep] = ppc64le_branchi;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqc].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqc].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cequc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cequc].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_cequc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqs] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqs].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqs].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cequs] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cequs].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_cequs].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqi] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqi].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqi].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cequ] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cequ].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_cequ].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cequl] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cequl].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_cequl].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_ceql] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceql].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceql].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqp] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqp].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqp].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqd] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqd].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqd].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_ceqf] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_ceqf].data1 = dill_eq_code;
  ppc64le_jump_table->c_data[dill_jmp_ceqf].data2 = DILL_F;
  ppc64le_jump_table->jmp_c[dill_jmp_cgec] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgec].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgec].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cgeuc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgeuc].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgeuc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_cges] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cges].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cges].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cgeus] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgeus].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgeus].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_cgei] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgei].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgei].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cgeu] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgeu].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgeu].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cgeul] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgeul].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgeul].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_cgel] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgel].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgel].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_cgep] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgep].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgep].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_cged] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cged].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cged].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_cgef] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgef].data1 = dill_ge_code;
  ppc64le_jump_table->c_data[dill_jmp_cgef].data2 = DILL_F;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtc].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtc].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtuc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtuc].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtuc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_cgts] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgts].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgts].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtus] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtus].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtus].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_cgti] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgti].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgti].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtu] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtu].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtu].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtul] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtul].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtul].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtl] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtl].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtl].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtp] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtp].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtp].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtd] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtd].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtd].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_cgtf] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cgtf].data1 = dill_gt_code;
  ppc64le_jump_table->c_data[dill_jmp_cgtf].data2 = DILL_F;
  ppc64le_jump_table->jmp_c[dill_jmp_clec] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clec].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_clec].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cleuc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cleuc].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cleuc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_cles] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cles].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cles].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cleus] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cleus].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cleus].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_clei] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clei].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_clei].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cleu] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cleu].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cleu].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cleul] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cleul].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cleul].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_clel] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clel].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_clel].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_clep] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clep].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_clep].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_cled] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cled].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_cled].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_clef] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clef].data1 = dill_le_code;
  ppc64le_jump_table->c_data[dill_jmp_clef].data2 = DILL_F;
  ppc64le_jump_table->jmp_c[dill_jmp_cltc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltc].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltc].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cltuc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltuc].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltuc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_clts] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clts].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_clts].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cltus] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltus].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltus].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_clti] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_clti].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_clti].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cltu] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltu].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltu].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cltul] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltul].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltul].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_cltl] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltl].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltl].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_cltp] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltp].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltp].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_cltd] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltd].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltd].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_cltf] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cltf].data1 = dill_lt_code;
  ppc64le_jump_table->c_data[dill_jmp_cltf].data2 = DILL_F;
  ppc64le_jump_table->jmp_c[dill_jmp_cnec] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnec].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnec].data2 = DILL_C;
  ppc64le_jump_table->jmp_c[dill_jmp_cneuc] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cneuc].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cneuc].data2 = DILL_UC;
  ppc64le_jump_table->jmp_c[dill_jmp_cnes] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnes].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnes].data2 = DILL_S;
  ppc64le_jump_table->jmp_c[dill_jmp_cneus] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cneus].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cneus].data2 = DILL_US;
  ppc64le_jump_table->jmp_c[dill_jmp_cnei] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnei].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnei].data2 = DILL_I;
  ppc64le_jump_table->jmp_c[dill_jmp_cneu] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cneu].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cneu].data2 = DILL_U;
  ppc64le_jump_table->jmp_c[dill_jmp_cneul] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cneul].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cneul].data2 = DILL_UL;
  ppc64le_jump_table->jmp_c[dill_jmp_cnel] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnel].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnel].data2 = DILL_L;
  ppc64le_jump_table->jmp_c[dill_jmp_cnep] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnep].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnep].data2 = DILL_P;
  ppc64le_jump_table->jmp_c[dill_jmp_cned] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cned].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cned].data2 = DILL_D;
  ppc64le_jump_table->jmp_c[dill_jmp_cnef] = ppc64le_compare;
  ppc64le_jump_table->c_data[dill_jmp_cnef].data1 = dill_ne_code;
  ppc64le_jump_table->c_data[dill_jmp_cnef].data2 = DILL_F;
  ppc64le_jump_table->jmp_ci[dill_jmp_ceqc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cequc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_ceqs] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cequs] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_ceqi] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cequ] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cequl] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_ceql] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_ceqp] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgec] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgeuc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cges] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgeus] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgei] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgeu] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgeul] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgel] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgep] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtuc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgts] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtus] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgti] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtu] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtul] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtl] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cgtp] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clec] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cleuc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cles] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cleus] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clei] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cleu] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cleul] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clel] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clep] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltuc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clts] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltus] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_clti] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltu] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltul] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltl] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cltp] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cnec] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cneuc] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cnes] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cneus] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cnei] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cneu] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cneul] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cnel] = ppc64le_comparei;
  ppc64le_jump_table->jmp_ci[dill_jmp_cnep] = ppc64le_comparei;
    ppc64le_jump_table->proc_start = (dill_mach_proc_start) ppc64le_proc_start;
    ppc64le_jump_table->end = ppc64le_end;
    ppc64le_jump_table->package_end = ppc64le_package_end;
    ppc64le_jump_table->clone_code = ppc64le_clone_code;
    ppc64le_jump_table->type_align = ppc64le_type_align;
    ppc64le_jump_table->type_size = ppc64le_type_size;
    ppc64le_jump_table->ret = ppc64le_ret;
    ppc64le_jump_table->reti = ppc64le_reti;
    ppc64le_jump_table->retf = (ret_opf)ppc64le_reti;
    ppc64le_jump_table->load = ppc64le_pload;
    ppc64le_jump_table->bsload = ppc64le_pbsload;
    ppc64le_jump_table->loadi = ppc64le_ploadi;
    ppc64le_jump_table->bsloadi = ppc64le_pbsloadi;
    ppc64le_jump_table->loadi = ppc64le_ploadi;
    ppc64le_jump_table->store = ppc64le_pstore;
    ppc64le_jump_table->storei = ppc64le_pstorei;
    ppc64le_jump_table->convert = ppc64le_convert;
    ppc64le_jump_table->mov = ppc64le_mov;
    ppc64le_jump_table->set = ppc64le_pset;
    ppc64le_jump_table->setf = ppc64le_setf;
    ppc64le_jump_table->setp = ppc64le_setp;
    ppc64le_jump_table->jv = ppc64le_jump_to_label;
    ppc64le_jump_table->jp = ppc64le_jump_to_reg;
    ppc64le_jump_table->jpi = ppc64le_jump_to_imm;
    ppc64le_jump_table->jal = ppc64le_jal;
    ppc64le_jump_table->push = ppc64le_push;
    ppc64le_jump_table->pushi = ppc64le_pushi;
    ppc64le_jump_table->pushfi = ppc64le_pushfi;
    ppc64le_jump_table->pushpi = ppc64le_pushpi;
    ppc64le_jump_table->calli = ppc64le_calli;
    ppc64le_jump_table->callr = ppc64le_callr;
    ppc64le_jump_table->local = ppc64le_local_op;
    ppc64le_jump_table->save_restore = ppc64le_save_restore_op;
    ppc64le_jump_table->init_disassembly = ppc64le_init_disassembly_info;
    ppc64le_jump_table->print_insn = ppc64le_print_insn;
    ppc64le_jump_table->print_reg = ppc64le_print_reg;
    ppc64le_jump_table->count_insn = ppc64le_count_insn;
    ppc64le_jump_table->do_reverse_push = 0;
    ppc64le_jump_table->target_byte_order = 2;  /* Format_Integer_littleendian */
    ppc64le_jump_table->target_float_format = 3;  /* Format_IEEE_754_littleendian */ 
}

extern void dill_ppc64le_init(dill_stream s)
{
        gen_ppc64le_jump_table();
    s->j = ppc64le_jump_table;
    s->dill_local_pointer = _sp;
    s->dill_param_reg_pointer = _sp;
    s->p->mach_info = (void*)gen_ppc64le_mach_info(s,1);
    s->p->machine_strr_tmp_reg = _gpr0;
    return;
}
