/*
 * SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include "foo.h"

int main(int argc, char **argv)
{
    foo();
    return 0;
}
