/*
 *  This file is automatically generated by gen_interface.pl from evpath.h.
 *
 *  DO NOT EDIT
 *
 */


extern int
INT_REVaction_add_split_target(CMConnection conn, EVstone stone, EVaction action, EVstone target_stone);

extern void
INT_REVaction_remove_split_target(CMConnection conn, EVstone stone, EVaction action, EVstone target_stone);

extern int
INT_REVaction_set_output(CMConnection conn, EVstone stone, EVaction action, int output_index, EVstone target_stone);

extern EVstone
INT_REValloc_stone(CMConnection conn);

extern EVaction
INT_REVassoc_bridge_action(CMConnection conn, EVstone stone, attr_list contact_list, EVstone remote_stone);

extern EVaction
INT_REVassoc_filter_action(CMConnection conn, EVstone stone, FMStructDescList incoming_format_list, char *handler, EVstone out_stone);

extern EVaction
INT_REVassoc_immediate_action(CMConnection conn, EVstone stone, char *action_spec);

extern EVaction
INT_REVassoc_multi_action(CMConnection conn, EVstone stone, char *action_spec);

extern EVaction
INT_REVassoc_split_action(CMConnection conn, EVstone stone, EVstone *target_list);

extern EVaction
INT_REVassoc_store_action(CMConnection conn, EVstone stone_num, EVstone out_stone, int store_limit);

extern EVaction
INT_REVassoc_terminal_action(CMConnection conn, EVstone stone, FMStructDescList format_list, char *handler);

extern void
INT_REVclear_stored(CMConnection conn, EVstone stone_num, EVaction action_num);

extern EVstone
INT_REVcreate_auto_stone(CMConnection conn, int period_sec, int period_usec, char *action_spec, EVstone out_stone);

extern EVstone
INT_REVcreate_bridge_action(CMConnection conn, attr_list contact_list, EVstone remote_stone);

extern EVstone
INT_REVcreate_immediate_action(CMConnection conn, char *action_spec, EVstone *target_list);

extern EVstone
INT_REVcreate_split_action(CMConnection conn, EVstone *target_list);

extern EVstone
INT_REVcreate_stone_action(CMConnection conn, char *action_spec);

extern EVstone
INT_REVcreate_store_action(CMConnection conn, EVstone out_stone, int store_limit);

extern EVstone
INT_REVcreate_terminal_action(CMConnection conn, FMStructDescList format_list, char *handler);

extern int
INT_REVdestroy_stone(CMConnection conn, EVstone stone_id);

extern int
INT_REVdrain_stone(CMConnection conn, EVstone stone_id);

extern void
INT_REVenable_auto_stone(CMConnection conn, EVstone stone_num, int period_sec, int period_usec);

extern attr_list
INT_REVextract_attr_list(CMConnection conn, EVstone stone_id);

extern void
INT_REVfree_stone(CMConnection conn, EVstone stone);

extern int
INT_REVfreeze_stone(CMConnection conn, EVstone stone_id);

extern void
INT_REVset_attr_list(CMConnection conn, EVstone stone_id, attr_list list);

extern void
INT_REVstall_stone(CMConnection conn, EVstone stone_id);

extern void
INT_REVstone_add_split_target(CMConnection conn, EVstone stone, EVstone target_stone);

extern void
INT_REVstone_remove_split_target(CMConnection conn, EVstone stone, EVstone target_stone);

extern int
INT_REVstone_set_output(CMConnection conn, EVstone stone, int output_index, EVstone target_stone);

extern int
INT_REVtransfer_events(CMConnection conn, EVstone src_stone, EVstone dest_stone);

extern int
INT_REVunfreeze_stone(CMConnection conn, EVstone stone_id);

extern void
INT_REVunstall_stone(CMConnection conn, EVstone stone_id);
