/* This file is generated from powerpc.ops.  Do not edit directly. */

#include "config.h"
#include "dill.h"
#include "dill_internal.h"
#include "powerpc.h"
#include <string.h>
extern void powerpc_FORM3_farith(s, op3, fop, dest, src1, src2)
dill_stream s;
int op3;
int fop;
int dest;
int src1;
int src2;
{
    /* format 3 */
    INSN_OUT(s, HDR(0x2)|OP3(op3)|RD(dest)|RS1(src1)|OPF(fop)|RS2(src2));
}

extern void powerpc_FORM2_farith(s, op3, fop, dest, src)
dill_stream s;
int op3;
int fop;
int dest;
int src;
{
    /* format 3 */
    INSN_OUT(s, HDR(0x2)|OP3(op3)|RD(dest)|OPF(fop)|RS2(src));
}

extern void powerpc_FORM2_arith(s, op3, op, dest, src)
dill_stream s;
int op3;
int op;
int dest;
int src;
{
    if (op != 0) {
	/* format 3 with g0 as rs1*/
	INSN_OUT(s, HDR(op)|OP3(op3)|RD(dest)|RS1(_g0)|RS2(src));
    } else {
	/* must be NOT */
	INSN_OUT(s, HDR(0x2)|RD(_g0)|OP3(0x14)|RS1(_g0)|RS2(src));/* subcc */
	INSN_OUT(s, HDR(0x2)|RD(dest)|OP3(0xc)|RS1(_g0)|IM|SIMM13(-1));
    }
}

static struct jmp_table_s powerpc_jump_table_struct;
static arith_op3 powerpc_arith_op3_table[dill_jmp_a3_size + 1];
static jmp_data  powerpc_arith_op3_data[dill_jmp_a3_size + 1];
static arith_op3i powerpc_arith_op3i_table[dill_jmp_a3_size + 1];
static jmp_data  powerpc_arith_op3i_data[dill_jmp_a3_size + 1];
static arith_op2 powerpc_arith_op2_table[dill_jmp_a2_size + 1];
static jmp_data  powerpc_arith_op2_data[dill_jmp_a2_size + 1];
static branch_op powerpc_branch_table[dill_jmp_branch_size + 1];
static branch_opi powerpc_branchi_table[dill_jmp_branch_size + 1];
static jmp_data  powerpc_branch_data[dill_jmp_branch_size + 1];
static compare_op powerpc_compare_table[dill_jmp_compare_size + 1];
static compare_opi powerpc_comparei_table[dill_jmp_compare_size + 1];
static jmp_data powerpc_compare_data[dill_jmp_compare_size + 1];

jmp_table powerpc_jump_table;
static void gen_powerpc_jump_table()
{
    powerpc_jump_table = &powerpc_jump_table_struct;          
    memset(powerpc_jump_table, 0, sizeof(struct jmp_table_s));
    /* arith3 */
    powerpc_jump_table->jmp_a3 = powerpc_arith_op3_table;
    memset(powerpc_jump_table->jmp_a3, 0, sizeof(powerpc_arith_op3_table));
    powerpc_jump_table->a3_data = powerpc_arith_op3_data;
    memset(powerpc_jump_table->a3_data, 0, sizeof(powerpc_arith_op3_data));
    /* arith3i */
    powerpc_jump_table->jmp_a3i = powerpc_arith_op3i_table;
    memset(powerpc_jump_table->jmp_a3i, 0, sizeof(powerpc_arith_op3i_table));
    powerpc_jump_table->a3i_data = powerpc_arith_op3i_data;
    memset(powerpc_jump_table->a3i_data, 0, sizeof(powerpc_arith_op3i_data));
    /* arith2 */
    powerpc_jump_table->jmp_a2 = powerpc_arith_op2_table;
    memset(powerpc_jump_table->jmp_a2, 0, sizeof(powerpc_arith_op2_table));
    powerpc_jump_table->a2_data = powerpc_arith_op2_data;
    memset(powerpc_jump_table->a2_data, 0, sizeof(powerpc_arith_op2_data));
    /* branch */
    powerpc_jump_table->jmp_b = powerpc_branch_table;
    memset(powerpc_jump_table->jmp_b, 0, sizeof(powerpc_branch_table));
    /* branchi */
    powerpc_jump_table->jmp_bi = powerpc_branchi_table;
    memset(powerpc_jump_table->jmp_bi, 0, sizeof(powerpc_branch_table));
    powerpc_jump_table->b_data = powerpc_branch_data;
    memset(powerpc_jump_table->b_data, 0, sizeof(powerpc_branch_data));
    /* compare */
    powerpc_jump_table->jmp_c = powerpc_compare_table;
    memset(powerpc_jump_table->jmp_c, 0, sizeof(powerpc_compare_table));
    /* comparei */
    powerpc_jump_table->jmp_ci = powerpc_comparei_table;
    memset(powerpc_jump_table->jmp_ci, 0, sizeof(powerpc_compare_table));
    powerpc_jump_table->c_data = powerpc_compare_data;
    memset(powerpc_jump_table->c_data, 0, sizeof(powerpc_compare_data));
    powerpc_jump_table->jmp_a3[dill_jmp_addi] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_addi].data1 = 0x0;
    powerpc_jump_table->a3_data[dill_jmp_addi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_addu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_addu].data1 = 0x0;
    powerpc_jump_table->a3_data[dill_jmp_addu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_addul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_addul].data1 = 0x0;
    powerpc_jump_table->a3_data[dill_jmp_addul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_addl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_addl].data1 = 0x0;
    powerpc_jump_table->a3_data[dill_jmp_addl].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_addp] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_addp].data1 = 0x0;
    powerpc_jump_table->a3_data[dill_jmp_addp].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_subi] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_subi].data1 = 0x4;
    powerpc_jump_table->a3_data[dill_jmp_subi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_subu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_subu].data1 = 0x4;
    powerpc_jump_table->a3_data[dill_jmp_subu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_subul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_subul].data1 = 0x4;
    powerpc_jump_table->a3_data[dill_jmp_subul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_subl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_subl].data1 = 0x4;
    powerpc_jump_table->a3_data[dill_jmp_subl].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_subp] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_subp].data1 = 0x4;
    powerpc_jump_table->a3_data[dill_jmp_subp].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_mulu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_mulu].data1 = 0x9/*umul*/;
    powerpc_jump_table->a3_data[dill_jmp_mulu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_mulul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_mulul].data1 = 0x9/*umul*/;
    powerpc_jump_table->a3_data[dill_jmp_mulul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_muli] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_muli].data1 = 0x9/*smul*/;
    powerpc_jump_table->a3_data[dill_jmp_muli].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_mull] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_mull].data1 = 0x9/*smul*/;
    powerpc_jump_table->a3_data[dill_jmp_mull].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_divu] = powerpc_div;
    powerpc_jump_table->a3_data[dill_jmp_divu].data1 = 0x0d/*udiv*/;
    powerpc_jump_table->a3_data[dill_jmp_divu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_divul] = powerpc_div;
    powerpc_jump_table->a3_data[dill_jmp_divul].data1 = 0x0d/*udiv*/;
    powerpc_jump_table->a3_data[dill_jmp_divul].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_divi] = powerpc_div;
    powerpc_jump_table->a3_data[dill_jmp_divi].data1 = 0x2d/*sdivx*/;
    powerpc_jump_table->a3_data[dill_jmp_divi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_divl] = powerpc_div;
    powerpc_jump_table->a3_data[dill_jmp_divl].data1 = 0x2d/*sdivx*/;
    powerpc_jump_table->a3_data[dill_jmp_divl].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_modu] = powerpc_mod;
    powerpc_jump_table->a3_data[dill_jmp_modu].data1 = 0x0/*umod*/;
    powerpc_jump_table->a3_data[dill_jmp_modu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_modul] = powerpc_mod;
    powerpc_jump_table->a3_data[dill_jmp_modul].data1 = 0x0/*umod*/;
    powerpc_jump_table->a3_data[dill_jmp_modul].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_modi] = powerpc_mod;
    powerpc_jump_table->a3_data[dill_jmp_modi].data1 = 0x1/*smod*/;
    powerpc_jump_table->a3_data[dill_jmp_modi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_modl] = powerpc_mod;
    powerpc_jump_table->a3_data[dill_jmp_modl].data1 = 0x1/*smod*/;
    powerpc_jump_table->a3_data[dill_jmp_modl].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_andi] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_andi].data1 = 0x1;
    powerpc_jump_table->a3_data[dill_jmp_andi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_andu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_andu].data1 = 0x1;
    powerpc_jump_table->a3_data[dill_jmp_andu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_andul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_andul].data1 = 0x1;
    powerpc_jump_table->a3_data[dill_jmp_andul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_andl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_andl].data1 = 0x1;
    powerpc_jump_table->a3_data[dill_jmp_andl].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_ori] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_ori].data1 = 0x2;
    powerpc_jump_table->a3_data[dill_jmp_ori].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_oru] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_oru].data1 = 0x2;
    powerpc_jump_table->a3_data[dill_jmp_oru].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_orul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_orul].data1 = 0x2;
    powerpc_jump_table->a3_data[dill_jmp_orul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_orl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_orl].data1 = 0x2;
    powerpc_jump_table->a3_data[dill_jmp_orl].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_xori] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_xori].data1 = 0x3;
    powerpc_jump_table->a3_data[dill_jmp_xori].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_xoru] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_xoru].data1 = 0x3;
    powerpc_jump_table->a3_data[dill_jmp_xoru].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_xorul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_xorul].data1 = 0x3;
    powerpc_jump_table->a3_data[dill_jmp_xorul].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_xorl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_xorl].data1 = 0x3;
    powerpc_jump_table->a3_data[dill_jmp_xorl].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_lshi] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_lshi].data1 = 0x25;
    powerpc_jump_table->a3_data[dill_jmp_lshi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_lshu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_lshu].data1 = 0x25;
    powerpc_jump_table->a3_data[dill_jmp_lshu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_lshul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_lshul].data1 = 0x25;
    powerpc_jump_table->a3_data[dill_jmp_lshul].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_lshl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_lshl].data1 = 0x25;
    powerpc_jump_table->a3_data[dill_jmp_lshl].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_rshi] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_rshi].data1 = 0x27;
    powerpc_jump_table->a3_data[dill_jmp_rshi].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_rshl] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_rshl].data1 = 0x27;
    powerpc_jump_table->a3_data[dill_jmp_rshl].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_rshu] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_rshu].data1 = 0x26;
    powerpc_jump_table->a3_data[dill_jmp_rshu].data2 = 0;
    powerpc_jump_table->jmp_a3[dill_jmp_rshul] = powerpc_FORM3_arith;
    powerpc_jump_table->a3_data[dill_jmp_rshul].data1 = 0x26;
    powerpc_jump_table->a3_data[dill_jmp_rshul].data2 = 1;
    powerpc_jump_table->jmp_a3[dill_jmp_addf] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_addf].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_addf].data2 = 0x041;
    powerpc_jump_table->jmp_a3[dill_jmp_addd] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_addd].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_addd].data2 = 0x042;
    powerpc_jump_table->jmp_a3[dill_jmp_subf] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_subf].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_subf].data2 = 0x045;
    powerpc_jump_table->jmp_a3[dill_jmp_subd] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_subd].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_subd].data2 = 0x046;
    powerpc_jump_table->jmp_a3[dill_jmp_mulf] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_mulf].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_mulf].data2 = 0x049;
    powerpc_jump_table->jmp_a3[dill_jmp_muld] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_muld].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_muld].data2 = 0x04a;
    powerpc_jump_table->jmp_a3[dill_jmp_divf] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_divf].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_divf].data2 = 0x04d;
    powerpc_jump_table->jmp_a3[dill_jmp_divd] = powerpc_FORM3_farith;
    powerpc_jump_table->a3_data[dill_jmp_divd].data1 = 0x34;
    powerpc_jump_table->a3_data[dill_jmp_divd].data2 = 0x04e;
    powerpc_jump_table->jmp_a3i[dill_jmp_addi] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_addi].data1 = 0x0;
    powerpc_jump_table->a3i_data[dill_jmp_addi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_addu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_addu].data1 = 0x0;
    powerpc_jump_table->a3i_data[dill_jmp_addu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_addul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_addul].data1 = 0x0;
    powerpc_jump_table->a3i_data[dill_jmp_addul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_addl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_addl].data1 = 0x0;
    powerpc_jump_table->a3i_data[dill_jmp_addl].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_addp] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_addp].data1 = 0x0;
    powerpc_jump_table->a3i_data[dill_jmp_addp].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_subi] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_subi].data1 = 0x4;
    powerpc_jump_table->a3i_data[dill_jmp_subi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_subu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_subu].data1 = 0x4;
    powerpc_jump_table->a3i_data[dill_jmp_subu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_subul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_subul].data1 = 0x4;
    powerpc_jump_table->a3i_data[dill_jmp_subul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_subl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_subl].data1 = 0x4;
    powerpc_jump_table->a3i_data[dill_jmp_subl].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_subp] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_subp].data1 = 0x4;
    powerpc_jump_table->a3i_data[dill_jmp_subp].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_mulu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_mulu].data1 = 0x9/*umul*/;
    powerpc_jump_table->a3i_data[dill_jmp_mulu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_mulul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_mulul].data1 = 0x9/*umul*/;
    powerpc_jump_table->a3i_data[dill_jmp_mulul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_muli] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_muli].data1 = 0x9/*smul*/;
    powerpc_jump_table->a3i_data[dill_jmp_muli].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_mull] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_mull].data1 = 0x9/*smul*/;
    powerpc_jump_table->a3i_data[dill_jmp_mull].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_divu] = powerpc_divi;
    powerpc_jump_table->a3i_data[dill_jmp_divu].data1 = 0x0d/*udiv*/;
    powerpc_jump_table->a3i_data[dill_jmp_divu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_divul] = powerpc_divi;
    powerpc_jump_table->a3i_data[dill_jmp_divul].data1 = 0x0d/*udiv*/;
    powerpc_jump_table->a3i_data[dill_jmp_divul].data2 = 1;
    powerpc_jump_table->jmp_a3i[dill_jmp_divi] = powerpc_divi;
    powerpc_jump_table->a3i_data[dill_jmp_divi].data1 = 0x2d/*sdiv*/;
    powerpc_jump_table->a3i_data[dill_jmp_divi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_divl] = powerpc_divi;
    powerpc_jump_table->a3i_data[dill_jmp_divl].data1 = 0x2d/*sdiv*/;
    powerpc_jump_table->a3i_data[dill_jmp_divl].data2 = 1;
    powerpc_jump_table->jmp_a3i[dill_jmp_modu] = powerpc_modi;
    powerpc_jump_table->a3i_data[dill_jmp_modu].data1 = 0x0/*umod*/;
    powerpc_jump_table->a3i_data[dill_jmp_modu].data2 = 2;
    powerpc_jump_table->jmp_a3i[dill_jmp_modul] = powerpc_modi;
    powerpc_jump_table->a3i_data[dill_jmp_modul].data1 = 0x0/*umod*/;
    powerpc_jump_table->a3i_data[dill_jmp_modul].data2 = 2;
    powerpc_jump_table->jmp_a3i[dill_jmp_modi] = powerpc_modi;
    powerpc_jump_table->a3i_data[dill_jmp_modi].data1 = 0x1/*smod*/;
    powerpc_jump_table->a3i_data[dill_jmp_modi].data2 = 2;
    powerpc_jump_table->jmp_a3i[dill_jmp_modl] = powerpc_modi;
    powerpc_jump_table->a3i_data[dill_jmp_modl].data1 = 0x1/*smod*/;
    powerpc_jump_table->a3i_data[dill_jmp_modl].data2 = 2;
    powerpc_jump_table->jmp_a3i[dill_jmp_andi] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_andi].data1 = 0x1;
    powerpc_jump_table->a3i_data[dill_jmp_andi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_andu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_andu].data1 = 0x1;
    powerpc_jump_table->a3i_data[dill_jmp_andu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_andul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_andul].data1 = 0x1;
    powerpc_jump_table->a3i_data[dill_jmp_andul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_andl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_andl].data1 = 0x1;
    powerpc_jump_table->a3i_data[dill_jmp_andl].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_ori] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_ori].data1 = 0x2;
    powerpc_jump_table->a3i_data[dill_jmp_ori].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_oru] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_oru].data1 = 0x2;
    powerpc_jump_table->a3i_data[dill_jmp_oru].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_orul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_orul].data1 = 0x2;
    powerpc_jump_table->a3i_data[dill_jmp_orul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_orl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_orl].data1 = 0x2;
    powerpc_jump_table->a3i_data[dill_jmp_orl].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_xori] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_xori].data1 = 0x3;
    powerpc_jump_table->a3i_data[dill_jmp_xori].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_xoru] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_xoru].data1 = 0x3;
    powerpc_jump_table->a3i_data[dill_jmp_xoru].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_xorul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_xorul].data1 = 0x3;
    powerpc_jump_table->a3i_data[dill_jmp_xorul].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_xorl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_xorl].data1 = 0x3;
    powerpc_jump_table->a3i_data[dill_jmp_xorl].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_lshi] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_lshi].data1 = 0x25;
    powerpc_jump_table->a3i_data[dill_jmp_lshi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_lshu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_lshu].data1 = 0x25;
    powerpc_jump_table->a3i_data[dill_jmp_lshu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_lshul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_lshul].data1 = 0x25;
    powerpc_jump_table->a3i_data[dill_jmp_lshul].data2 = 1;
    powerpc_jump_table->jmp_a3i[dill_jmp_lshl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_lshl].data1 = 0x25;
    powerpc_jump_table->a3i_data[dill_jmp_lshl].data2 = 1;
    powerpc_jump_table->jmp_a3i[dill_jmp_rshi] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_rshi].data1 = 0x27;
    powerpc_jump_table->a3i_data[dill_jmp_rshi].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_rshl] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_rshl].data1 = 0x27;
    powerpc_jump_table->a3i_data[dill_jmp_rshl].data2 = 1;
    powerpc_jump_table->jmp_a3i[dill_jmp_rshu] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_rshu].data1 = 0x26;
    powerpc_jump_table->a3i_data[dill_jmp_rshu].data2 = 0;
    powerpc_jump_table->jmp_a3i[dill_jmp_rshul] = powerpc_FORM3imm_arith;
    powerpc_jump_table->a3i_data[dill_jmp_rshul].data1 = 0x26;
    powerpc_jump_table->a3i_data[dill_jmp_rshul].data2 = 1;
    powerpc_jump_table->jmp_a2[dill_jmp_comi] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_comi].data1 = 0x07;
    powerpc_jump_table->a2_data[dill_jmp_comi].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_comu] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_comu].data1 = 0x07;
    powerpc_jump_table->a2_data[dill_jmp_comu].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_comul] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_comul].data1 = 0x07;
    powerpc_jump_table->a2_data[dill_jmp_comul].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_coml] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_coml].data1 = 0x07;
    powerpc_jump_table->a2_data[dill_jmp_coml].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_negi] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_negi].data1 = 0x04;
    powerpc_jump_table->a2_data[dill_jmp_negi].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_negu] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_negu].data1 = 0x04;
    powerpc_jump_table->a2_data[dill_jmp_negu].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_negul] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_negul].data1 = 0x04;
    powerpc_jump_table->a2_data[dill_jmp_negul].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_negl] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_negl].data1 = 0x04;
    powerpc_jump_table->a2_data[dill_jmp_negl].data2 = 2;
    powerpc_jump_table->jmp_a2[dill_jmp_noti] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_noti].data1 = dill_jmp_noti;
    powerpc_jump_table->a2_data[dill_jmp_noti].data2 = 0;
    powerpc_jump_table->jmp_a2[dill_jmp_notu] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_notu].data1 = dill_jmp_noti;
    powerpc_jump_table->a2_data[dill_jmp_notu].data2 = 0;
    powerpc_jump_table->jmp_a2[dill_jmp_notul] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_notul].data1 = dill_jmp_noti;
    powerpc_jump_table->a2_data[dill_jmp_notul].data2 = 0;
    powerpc_jump_table->jmp_a2[dill_jmp_notl] = (arith_op2)powerpc_FORM2_arith;
    powerpc_jump_table->a2_data[dill_jmp_notl].data1 = dill_jmp_noti;
    powerpc_jump_table->a2_data[dill_jmp_notl].data2 = 0;
    powerpc_jump_table->jmp_a2[dill_jmp_negf] = powerpc_FORM2_farith;
    powerpc_jump_table->a2_data[dill_jmp_negf].data1 = 0x34;
    powerpc_jump_table->a2_data[dill_jmp_negf].data2 = 0x05;
    powerpc_jump_table->jmp_a2[dill_jmp_negd] = powerpc_FORM2_farith;
    powerpc_jump_table->a2_data[dill_jmp_negd].data1 = 0x34;
    powerpc_jump_table->a2_data[dill_jmp_negd].data2 = 0x06;
    powerpc_jump_table->jmp_a2[dill_jmp_bswaps] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswaps].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswaps].data2 = DILL_S;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapus] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapus].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapus].data2 = DILL_US;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapi] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapi].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapi].data2 = DILL_I;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapu] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapu].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapu].data2 = DILL_U;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapul] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapul].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapul].data2 = DILL_UL;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapl] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapl].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapl].data2 = DILL_L;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapf] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapf].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapf].data2 = DILL_F;
    powerpc_jump_table->jmp_a2[dill_jmp_bswapd] = powerpc_bswap;
    powerpc_jump_table->a2_data[dill_jmp_bswapd].data1 = 0;
    powerpc_jump_table->a2_data[dill_jmp_bswapd].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_beqc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqc].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqc].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bequc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bequc].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_bequc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_beqs] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqs].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqs].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bequs] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bequs].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_bequs].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_beqi] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqi].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqi].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bequ] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bequ].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_bequ].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bequl] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bequl].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_bequl].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_beql] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beql].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beql].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_beqp] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqp].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqp].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_beqd] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqd].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqd].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_beqf] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_beqf].data1 = dill_eq_code;
    powerpc_jump_table->b_data[dill_jmp_beqf].data2 = DILL_F;
    powerpc_jump_table->jmp_b[dill_jmp_bgec] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgec].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgec].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bgeuc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgeuc].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgeuc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_bges] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bges].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bges].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bgeus] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgeus].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgeus].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_bgei] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgei].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgei].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bgeu] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgeu].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgeu].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bgeul] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgeul].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgeul].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_bgel] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgel].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgel].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_bgep] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgep].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgep].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_bged] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bged].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bged].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_bgef] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgef].data1 = dill_ge_code;
    powerpc_jump_table->b_data[dill_jmp_bgef].data2 = DILL_F;
    powerpc_jump_table->jmp_b[dill_jmp_bgtc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtc].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtc].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bgtuc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtuc].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtuc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_bgts] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgts].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgts].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bgtus] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtus].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtus].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_bgti] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgti].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgti].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bgtu] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtu].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtu].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bgtul] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtul].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtul].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_bgtl] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtl].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtl].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_bgtp] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtp].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtp].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_bgtd] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtd].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtd].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_bgtf] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bgtf].data1 = dill_gt_code;
    powerpc_jump_table->b_data[dill_jmp_bgtf].data2 = DILL_F;
    powerpc_jump_table->jmp_b[dill_jmp_blec] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blec].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_blec].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bleuc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bleuc].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bleuc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_bles] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bles].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bles].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bleus] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bleus].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bleus].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_blei] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blei].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_blei].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bleu] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bleu].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bleu].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bleul] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bleul].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bleul].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_blel] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blel].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_blel].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_blep] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blep].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_blep].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_bled] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bled].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_bled].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_blef] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blef].data1 = dill_le_code;
    powerpc_jump_table->b_data[dill_jmp_blef].data2 = DILL_F;
    powerpc_jump_table->jmp_b[dill_jmp_bltc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltc].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltc].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bltuc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltuc].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltuc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_blts] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blts].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_blts].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bltus] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltus].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltus].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_blti] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_blti].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_blti].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bltu] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltu].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltu].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bltul] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltul].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltul].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_bltl] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltl].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltl].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_bltp] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltp].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltp].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_bltd] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltd].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltd].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_bltf] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bltf].data1 = dill_lt_code;
    powerpc_jump_table->b_data[dill_jmp_bltf].data2 = DILL_F;
    powerpc_jump_table->jmp_b[dill_jmp_bnec] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnec].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnec].data2 = DILL_C;
    powerpc_jump_table->jmp_b[dill_jmp_bneuc] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bneuc].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bneuc].data2 = DILL_UC;
    powerpc_jump_table->jmp_b[dill_jmp_bnes] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnes].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnes].data2 = DILL_S;
    powerpc_jump_table->jmp_b[dill_jmp_bneus] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bneus].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bneus].data2 = DILL_US;
    powerpc_jump_table->jmp_b[dill_jmp_bnei] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnei].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnei].data2 = DILL_I;
    powerpc_jump_table->jmp_b[dill_jmp_bneu] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bneu].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bneu].data2 = DILL_U;
    powerpc_jump_table->jmp_b[dill_jmp_bneul] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bneul].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bneul].data2 = DILL_UL;
    powerpc_jump_table->jmp_b[dill_jmp_bnel] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnel].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnel].data2 = DILL_L;
    powerpc_jump_table->jmp_b[dill_jmp_bnep] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnep].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnep].data2 = DILL_P;
    powerpc_jump_table->jmp_b[dill_jmp_bned] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bned].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bned].data2 = DILL_D;
    powerpc_jump_table->jmp_b[dill_jmp_bnef] = powerpc_branch;
    powerpc_jump_table->b_data[dill_jmp_bnef].data1 = dill_ne_code;
    powerpc_jump_table->b_data[dill_jmp_bnef].data2 = DILL_F;
    powerpc_jump_table->jmp_bi[dill_jmp_beqc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bequc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_beqs] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bequs] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_beqi] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bequ] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bequl] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_beql] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_beqp] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgec] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgeuc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bges] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgeus] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgei] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgeu] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgeul] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgel] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgep] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtuc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgts] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtus] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgti] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtu] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtul] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtl] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bgtp] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blec] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bleuc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bles] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bleus] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blei] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bleu] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bleul] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blel] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blep] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltuc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blts] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltus] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_blti] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltu] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltul] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltl] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bltp] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bnec] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bneuc] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bnes] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bneus] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bnei] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bneu] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bneul] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bnel] = powerpc_branchi;
    powerpc_jump_table->jmp_bi[dill_jmp_bnep] = powerpc_branchi;
  powerpc_jump_table->jmp_c[dill_jmp_ceqc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqc].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqc].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cequc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cequc].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_cequc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_ceqs] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqs].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqs].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cequs] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cequs].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_cequs].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_ceqi] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqi].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqi].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cequ] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cequ].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_cequ].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cequl] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cequl].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_cequl].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_ceql] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceql].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceql].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_ceqp] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqp].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqp].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_ceqd] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqd].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqd].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_ceqf] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_ceqf].data1 = dill_eq_code;
  powerpc_jump_table->c_data[dill_jmp_ceqf].data2 = DILL_F;
  powerpc_jump_table->jmp_c[dill_jmp_cgec] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgec].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgec].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cgeuc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgeuc].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgeuc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_cges] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cges].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cges].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cgeus] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgeus].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgeus].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_cgei] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgei].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgei].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cgeu] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgeu].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgeu].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cgeul] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgeul].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgeul].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_cgel] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgel].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgel].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_cgep] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgep].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgep].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_cged] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cged].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cged].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_cgef] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgef].data1 = dill_ge_code;
  powerpc_jump_table->c_data[dill_jmp_cgef].data2 = DILL_F;
  powerpc_jump_table->jmp_c[dill_jmp_cgtc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtc].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtc].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cgtuc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtuc].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtuc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_cgts] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgts].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgts].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cgtus] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtus].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtus].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_cgti] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgti].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgti].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cgtu] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtu].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtu].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cgtul] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtul].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtul].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_cgtl] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtl].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtl].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_cgtp] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtp].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtp].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_cgtd] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtd].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtd].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_cgtf] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cgtf].data1 = dill_gt_code;
  powerpc_jump_table->c_data[dill_jmp_cgtf].data2 = DILL_F;
  powerpc_jump_table->jmp_c[dill_jmp_clec] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clec].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_clec].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cleuc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cleuc].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cleuc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_cles] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cles].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cles].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cleus] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cleus].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cleus].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_clei] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clei].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_clei].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cleu] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cleu].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cleu].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cleul] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cleul].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cleul].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_clel] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clel].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_clel].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_clep] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clep].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_clep].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_cled] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cled].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_cled].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_clef] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clef].data1 = dill_le_code;
  powerpc_jump_table->c_data[dill_jmp_clef].data2 = DILL_F;
  powerpc_jump_table->jmp_c[dill_jmp_cltc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltc].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltc].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cltuc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltuc].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltuc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_clts] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clts].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_clts].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cltus] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltus].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltus].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_clti] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_clti].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_clti].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cltu] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltu].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltu].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cltul] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltul].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltul].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_cltl] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltl].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltl].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_cltp] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltp].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltp].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_cltd] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltd].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltd].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_cltf] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cltf].data1 = dill_lt_code;
  powerpc_jump_table->c_data[dill_jmp_cltf].data2 = DILL_F;
  powerpc_jump_table->jmp_c[dill_jmp_cnec] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnec].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnec].data2 = DILL_C;
  powerpc_jump_table->jmp_c[dill_jmp_cneuc] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cneuc].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cneuc].data2 = DILL_UC;
  powerpc_jump_table->jmp_c[dill_jmp_cnes] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnes].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnes].data2 = DILL_S;
  powerpc_jump_table->jmp_c[dill_jmp_cneus] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cneus].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cneus].data2 = DILL_US;
  powerpc_jump_table->jmp_c[dill_jmp_cnei] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnei].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnei].data2 = DILL_I;
  powerpc_jump_table->jmp_c[dill_jmp_cneu] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cneu].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cneu].data2 = DILL_U;
  powerpc_jump_table->jmp_c[dill_jmp_cneul] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cneul].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cneul].data2 = DILL_UL;
  powerpc_jump_table->jmp_c[dill_jmp_cnel] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnel].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnel].data2 = DILL_L;
  powerpc_jump_table->jmp_c[dill_jmp_cnep] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnep].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnep].data2 = DILL_P;
  powerpc_jump_table->jmp_c[dill_jmp_cned] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cned].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cned].data2 = DILL_D;
  powerpc_jump_table->jmp_c[dill_jmp_cnef] = powerpc_compare;
  powerpc_jump_table->c_data[dill_jmp_cnef].data1 = dill_ne_code;
  powerpc_jump_table->c_data[dill_jmp_cnef].data2 = DILL_F;
  powerpc_jump_table->jmp_ci[dill_jmp_ceqc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cequc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_ceqs] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cequs] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_ceqi] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cequ] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cequl] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_ceql] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_ceqp] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgec] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgeuc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cges] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgeus] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgei] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgeu] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgeul] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgel] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgep] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtuc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgts] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtus] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgti] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtu] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtul] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtl] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cgtp] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clec] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cleuc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cles] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cleus] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clei] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cleu] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cleul] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clel] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clep] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltuc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clts] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltus] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_clti] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltu] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltul] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltl] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cltp] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cnec] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cneuc] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cnes] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cneus] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cnei] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cneu] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cneul] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cnel] = powerpc_comparei;
  powerpc_jump_table->jmp_ci[dill_jmp_cnep] = powerpc_comparei;
    powerpc_jump_table->proc_start = (dill_mach_proc_start) powerpc_proc_start;
    powerpc_jump_table->end = powerpc_end;
    powerpc_jump_table->package_end = powerpc_package_end;
    powerpc_jump_table->clone_code = powerpc_clone_code;
    powerpc_jump_table->type_align = powerpc_type_align;
    powerpc_jump_table->type_size = powerpc_type_size;
    powerpc_jump_table->ret = powerpc_ret;
    powerpc_jump_table->reti = powerpc_reti;
    powerpc_jump_table->retf = (ret_opf)powerpc_reti;
    powerpc_jump_table->load = powerpc_pload;
    powerpc_jump_table->bsload = powerpc_pbsload;
    powerpc_jump_table->loadi = powerpc_ploadi;
    powerpc_jump_table->bsloadi = powerpc_pbsloadi;
    powerpc_jump_table->loadi = powerpc_ploadi;
    powerpc_jump_table->store = powerpc_pstore;
    powerpc_jump_table->storei = powerpc_pstorei;
    powerpc_jump_table->convert = powerpc_convert;
    powerpc_jump_table->mov = powerpc_mov;
    powerpc_jump_table->set = powerpc_pset;
    powerpc_jump_table->setf = powerpc_setf;
    powerpc_jump_table->setp = powerpc_setp;
    powerpc_jump_table->jv = powerpc_jump_to_label;
    powerpc_jump_table->jp = powerpc_jump_to_reg;
    powerpc_jump_table->jpi = powerpc_jump_to_imm;
    powerpc_jump_table->jal = powerpc_jal;
    powerpc_jump_table->push = powerpc_push;
    powerpc_jump_table->pushi = powerpc_pushi;
    powerpc_jump_table->pushfi = powerpc_pushfi;
    powerpc_jump_table->pushpi = powerpc_pushpi;
    powerpc_jump_table->calli = powerpc_calli;
    powerpc_jump_table->callr = powerpc_callr;
    powerpc_jump_table->local = powerpc_local_op;
    powerpc_jump_table->save_restore = powerpc_save_restore_op;
    powerpc_jump_table->init_disassembly = powerpc_init_disassembly_info;
    powerpc_jump_table->print_insn = powerpc_print_insn;
    powerpc_jump_table->print_reg = powerpc_print_reg;
    powerpc_jump_table->count_insn = powerpc_count_insn;
    powerpc_jump_table->do_reverse_push = 0;
    powerpc_jump_table->target_byte_order = 1;  /* Format_Integer_bigendian */
    powerpc_jump_table->target_float_format = 1;  /* Format_IEEE_754_bigendian */ 
}

extern void dill_powerpc_init(dill_stream s)
{
    if(powerpc_jump_table == 0) {
        gen_powerpc_jump_table();
    }
    s->j = powerpc_jump_table;
    s->dill_local_pointer = _fp;
    s->dill_param_reg_pointer = _fp;
    s->p->mach_info = (void*)gen_powerpc_mach_info(s,/*v9*/ 0);
    s->p->machine_strr_tmp_reg = _g1;
    return;
}
extern void dill_powerpcv9_init(dill_stream s)
{
    if(powerpc_jump_table == 0) {
        gen_powerpc_jump_table();
    }
    s->j = powerpc_jump_table;
    s->dill_local_pointer = _fp;
    s->dill_param_reg_pointer = _fp;
    s->p->mach_info = (void*)gen_powerpc_mach_info(s,/*v9*/ 1);
    s->p->machine_strr_tmp_reg = _g1;
    return;
}
