/*
 * SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdbool.h>
#include <rdma/fabric.h>
#include <rdma/fi_cxi_ext.h>

int main() {}
