/* This file is generated from arm5.ops.  Do not edit directly. */

#include "dill.h"
#include "dill_internal.h"
#include "arm5.h"
DECLARE_JUMP_TABLE(arm5);
static void gen_arm5_jump_table()
{
	  FILL_JUMP_STRUCTURE(arm5);
	  arm5_jump_table->jmp_a3[dill_jmp_addi] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_addi].data1 = ADD;
	  arm5_jump_table->a3_data[dill_jmp_addi].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_addu] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_addu].data1 = ADD;
	  arm5_jump_table->a3_data[dill_jmp_addu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_addul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_addul].data1 = ADD;
	  arm5_jump_table->a3_data[dill_jmp_addul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_addl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_addl].data1 = ADD;
	  arm5_jump_table->a3_data[dill_jmp_addl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_addp] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_addp].data1 = ADD;
	  arm5_jump_table->a3_data[dill_jmp_addp].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_subi] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_subi].data1 = SUB;
	  arm5_jump_table->a3_data[dill_jmp_subi].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_subu] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_subu].data1 = SUB;
	  arm5_jump_table->a3_data[dill_jmp_subu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_subul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_subul].data1 = SUB;
	  arm5_jump_table->a3_data[dill_jmp_subul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_subl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_subl].data1 = SUB;
	  arm5_jump_table->a3_data[dill_jmp_subl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_subp] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_subp].data1 = SUB;
	  arm5_jump_table->a3_data[dill_jmp_subp].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_mulu] = arm5_mul;
	  arm5_jump_table->a3_data[dill_jmp_mulu].data1 = 1;
	  arm5_jump_table->a3_data[dill_jmp_mulu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_mulul] = arm5_mul;
	  arm5_jump_table->a3_data[dill_jmp_mulul].data1 = 1;
	  arm5_jump_table->a3_data[dill_jmp_mulul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_muli] = arm5_mul;
	  arm5_jump_table->a3_data[dill_jmp_muli].data1 = 0;
	  arm5_jump_table->a3_data[dill_jmp_muli].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_mull] = arm5_mul;
	  arm5_jump_table->a3_data[dill_jmp_mull].data1 = 0;
	  arm5_jump_table->a3_data[dill_jmp_mull].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_divu] = arm5_div;
	  arm5_jump_table->a3_data[dill_jmp_divu].data1 = 1;
	  arm5_jump_table->a3_data[dill_jmp_divu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_divul] = arm5_div;
	  arm5_jump_table->a3_data[dill_jmp_divul].data1 = 1;
	  arm5_jump_table->a3_data[dill_jmp_divul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_divi] = arm5_div;
	  arm5_jump_table->a3_data[dill_jmp_divi].data1 = 0;
	  arm5_jump_table->a3_data[dill_jmp_divi].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_divl] = arm5_div;
	  arm5_jump_table->a3_data[dill_jmp_divl].data1 = 0;
	  arm5_jump_table->a3_data[dill_jmp_divl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_modu] = arm5_mod;
	  arm5_jump_table->a3_data[dill_jmp_modu].data1 = 0x0/*umod*/;
	  arm5_jump_table->a3_data[dill_jmp_modu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_modul] = arm5_mod;
	  arm5_jump_table->a3_data[dill_jmp_modul].data1 = 0x0/*umod*/;
	  arm5_jump_table->a3_data[dill_jmp_modul].data2 = 1;
	  arm5_jump_table->jmp_a3[dill_jmp_modi] = arm5_mod;
	  arm5_jump_table->a3_data[dill_jmp_modi].data1 = 0x1/*smod*/;
	  arm5_jump_table->a3_data[dill_jmp_modi].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_modl] = arm5_mod;
	  arm5_jump_table->a3_data[dill_jmp_modl].data1 = 0x1/*smod*/;
	  arm5_jump_table->a3_data[dill_jmp_modl].data2 = 1;
	  arm5_jump_table->jmp_a3[dill_jmp_andi] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_andi].data1 = AND;
	  arm5_jump_table->a3_data[dill_jmp_andi].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_andu] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_andu].data1 = AND;
	  arm5_jump_table->a3_data[dill_jmp_andu].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_andul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_andul].data1 = AND;
	  arm5_jump_table->a3_data[dill_jmp_andul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_andl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_andl].data1 = AND;
	  arm5_jump_table->a3_data[dill_jmp_andl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_ori] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_ori].data1 = ORR;
	  arm5_jump_table->a3_data[dill_jmp_ori].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_oru] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_oru].data1 = ORR;
	  arm5_jump_table->a3_data[dill_jmp_oru].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_orul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_orul].data1 = ORR;
	  arm5_jump_table->a3_data[dill_jmp_orul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_orl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_orl].data1 = ORR;
	  arm5_jump_table->a3_data[dill_jmp_orl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_xori] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_xori].data1 = EOR;
	  arm5_jump_table->a3_data[dill_jmp_xori].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_xoru] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_xoru].data1 = EOR;
	  arm5_jump_table->a3_data[dill_jmp_xoru].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_xorul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_xorul].data1 = EOR;
	  arm5_jump_table->a3_data[dill_jmp_xorul].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_xorl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_xorl].data1 = EOR;
	  arm5_jump_table->a3_data[dill_jmp_xorl].data2 = 0;
	  arm5_jump_table->jmp_a3[dill_jmp_lshi] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_lshi].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_lshi].data2 = LLshift;
	  arm5_jump_table->jmp_a3[dill_jmp_lshu] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_lshu].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_lshu].data2 = LLshift;
	  arm5_jump_table->jmp_a3[dill_jmp_lshul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_lshul].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_lshul].data2 = LLshift;
	  arm5_jump_table->jmp_a3[dill_jmp_lshl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_lshl].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_lshl].data2 = LLshift;
	  arm5_jump_table->jmp_a3[dill_jmp_rshi] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_rshi].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_rshi].data2 = ARshift;
	  arm5_jump_table->jmp_a3[dill_jmp_rshl] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_rshl].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_rshl].data2 = ARshift;
	  arm5_jump_table->jmp_a3[dill_jmp_rshu] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_rshu].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_rshu].data2 = LRshift;
	  arm5_jump_table->jmp_a3[dill_jmp_rshul] = arm5_dproc;
	  arm5_jump_table->a3_data[dill_jmp_rshul].data1 = MOV;
	  arm5_jump_table->a3_data[dill_jmp_rshul].data2 = LRshift;
	  arm5_jump_table->jmp_a3[dill_jmp_addf] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_addf].data1 = 0x0;
	  arm5_jump_table->a3_data[dill_jmp_addf].data2 = 0x0;
	  arm5_jump_table->jmp_a3[dill_jmp_addd] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_addd].data1 = 0x0;
	  arm5_jump_table->a3_data[dill_jmp_addd].data2 = 0x1;
	  arm5_jump_table->jmp_a3[dill_jmp_subf] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_subf].data1 = 0x4;
	  arm5_jump_table->a3_data[dill_jmp_subf].data2 = 0x0;
	  arm5_jump_table->jmp_a3[dill_jmp_subd] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_subd].data1 = 0x4;
	  arm5_jump_table->a3_data[dill_jmp_subd].data2 = 0x1;
	  arm5_jump_table->jmp_a3[dill_jmp_mulf] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_mulf].data1 = 0x2;
	  arm5_jump_table->a3_data[dill_jmp_mulf].data2 = 0x0;
	  arm5_jump_table->jmp_a3[dill_jmp_muld] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_muld].data1 = 0x2;
	  arm5_jump_table->a3_data[dill_jmp_muld].data2 = 0x1;
	  arm5_jump_table->jmp_a3[dill_jmp_divf] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_divf].data1 = 0x8;
	  arm5_jump_table->a3_data[dill_jmp_divf].data2 = 0x0;
	  arm5_jump_table->jmp_a3[dill_jmp_divd] = arm5_fproc;
	  arm5_jump_table->a3_data[dill_jmp_divd].data1 = 0x8;
	  arm5_jump_table->a3_data[dill_jmp_divd].data2 = 0x1;
	  arm5_jump_table->jmp_a3i[dill_jmp_addi] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_addi].data1 = ADD;
	  arm5_jump_table->a3i_data[dill_jmp_addi].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_addu] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_addu].data1 = ADD;
	  arm5_jump_table->a3i_data[dill_jmp_addu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_addul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_addul].data1 = ADD;
	  arm5_jump_table->a3i_data[dill_jmp_addul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_addl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_addl].data1 = ADD;
	  arm5_jump_table->a3i_data[dill_jmp_addl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_addp] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_addp].data1 = ADD;
	  arm5_jump_table->a3i_data[dill_jmp_addp].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_subi] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_subi].data1 = SUB;
	  arm5_jump_table->a3i_data[dill_jmp_subi].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_subu] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_subu].data1 = SUB;
	  arm5_jump_table->a3i_data[dill_jmp_subu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_subul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_subul].data1 = SUB;
	  arm5_jump_table->a3i_data[dill_jmp_subul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_subl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_subl].data1 = SUB;
	  arm5_jump_table->a3i_data[dill_jmp_subl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_subp] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_subp].data1 = SUB;
	  arm5_jump_table->a3i_data[dill_jmp_subp].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_mulu] = arm5_muli;
	  arm5_jump_table->a3i_data[dill_jmp_mulu].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_mulu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_mulul] = arm5_muli;
	  arm5_jump_table->a3i_data[dill_jmp_mulul].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_mulul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_muli] = arm5_muli;
	  arm5_jump_table->a3i_data[dill_jmp_muli].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_muli].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_mull] = arm5_muli;
	  arm5_jump_table->a3i_data[dill_jmp_mull].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_mull].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_divu] = arm5_divi;
	  arm5_jump_table->a3i_data[dill_jmp_divu].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_divu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_divul] = arm5_divi;
	  arm5_jump_table->a3i_data[dill_jmp_divul].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_divul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_divi] = arm5_divi;
	  arm5_jump_table->a3i_data[dill_jmp_divi].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_divi].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_divl] = arm5_divi;
	  arm5_jump_table->a3i_data[dill_jmp_divl].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_divl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_modu] = arm5_modi;
	  arm5_jump_table->a3i_data[dill_jmp_modu].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_modu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_modul] = arm5_modi;
	  arm5_jump_table->a3i_data[dill_jmp_modul].data1 = 0;
	  arm5_jump_table->a3i_data[dill_jmp_modul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_modi] = arm5_modi;
	  arm5_jump_table->a3i_data[dill_jmp_modi].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_modi].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_modl] = arm5_modi;
	  arm5_jump_table->a3i_data[dill_jmp_modl].data1 = 1;
	  arm5_jump_table->a3i_data[dill_jmp_modl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_andi] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_andi].data1 = AND;
	  arm5_jump_table->a3i_data[dill_jmp_andi].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_andu] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_andu].data1 = AND;
	  arm5_jump_table->a3i_data[dill_jmp_andu].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_andul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_andul].data1 = AND;
	  arm5_jump_table->a3i_data[dill_jmp_andul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_andl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_andl].data1 = AND;
	  arm5_jump_table->a3i_data[dill_jmp_andl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_ori] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_ori].data1 = ORR;
	  arm5_jump_table->a3i_data[dill_jmp_ori].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_oru] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_oru].data1 = ORR;
	  arm5_jump_table->a3i_data[dill_jmp_oru].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_orul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_orul].data1 = ORR;
	  arm5_jump_table->a3i_data[dill_jmp_orul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_orl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_orl].data1 = ORR;
	  arm5_jump_table->a3i_data[dill_jmp_orl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_xori] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_xori].data1 = EOR;
	  arm5_jump_table->a3i_data[dill_jmp_xori].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_xoru] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_xoru].data1 = EOR;
	  arm5_jump_table->a3i_data[dill_jmp_xoru].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_xorul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_xorul].data1 = EOR;
	  arm5_jump_table->a3i_data[dill_jmp_xorul].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_xorl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_xorl].data1 = EOR;
	  arm5_jump_table->a3i_data[dill_jmp_xorl].data2 = 0;
	  arm5_jump_table->jmp_a3i[dill_jmp_lshi] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_lshi].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_lshi].data2 = LLshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_lshl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_lshl].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_lshl].data2 = LLshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_lshu] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_lshu].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_lshu].data2 = LLshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_lshul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_lshul].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_lshul].data2 = LLshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_rshi] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_rshi].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_rshi].data2 = ARshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_rshl] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_rshl].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_rshl].data2 = ARshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_rshu] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_rshu].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_rshu].data2 = LRshift;
	  arm5_jump_table->jmp_a3i[dill_jmp_rshul] = arm5_dproci;
	  arm5_jump_table->a3i_data[dill_jmp_rshul].data1 = MOV;
	  arm5_jump_table->a3i_data[dill_jmp_rshul].data2 = LRshift;
	  arm5_jump_table->jmp_a2[dill_jmp_comi] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_comi].data1 = MVN;
	  arm5_jump_table->a2_data[dill_jmp_comi].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_comu] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_comu].data1 = MVN;
	  arm5_jump_table->a2_data[dill_jmp_comu].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_comul] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_comul].data1 = MVN;
	  arm5_jump_table->a2_data[dill_jmp_comul].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_coml] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_coml].data1 = MVN;
	  arm5_jump_table->a2_data[dill_jmp_coml].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_negi] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_negi].data1 = RSB;
	  arm5_jump_table->a2_data[dill_jmp_negi].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_negu] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_negu].data1 = RSB;
	  arm5_jump_table->a2_data[dill_jmp_negu].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_negul] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_negul].data1 = RSB;
	  arm5_jump_table->a2_data[dill_jmp_negul].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_negl] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_negl].data1 = RSB;
	  arm5_jump_table->a2_data[dill_jmp_negl].data2 = 2;
	  arm5_jump_table->jmp_a2[dill_jmp_noti] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_noti].data1 = CMN;
	  arm5_jump_table->a2_data[dill_jmp_noti].data2 = 0;
	  arm5_jump_table->jmp_a2[dill_jmp_notu] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_notu].data1 = CMN;
	  arm5_jump_table->a2_data[dill_jmp_notu].data2 = 0;
	  arm5_jump_table->jmp_a2[dill_jmp_notul] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_notul].data1 = CMN;
	  arm5_jump_table->a2_data[dill_jmp_notul].data2 = 0;
	  arm5_jump_table->jmp_a2[dill_jmp_notl] = (arith_op2)arm5_dproc2;
	  arm5_jump_table->a2_data[dill_jmp_notl].data1 = CMN;
	  arm5_jump_table->a2_data[dill_jmp_notl].data2 = 0;
	  arm5_jump_table->jmp_a2[dill_jmp_negf] = arm5_fproc2;
	  arm5_jump_table->a2_data[dill_jmp_negf].data1 = 0x3;
	  arm5_jump_table->a2_data[dill_jmp_negf].data2 = 0x0;
	  arm5_jump_table->jmp_a2[dill_jmp_negd] = arm5_fproc2;
	  arm5_jump_table->a2_data[dill_jmp_negd].data1 = 0x3;
	  arm5_jump_table->a2_data[dill_jmp_negd].data2 = 0x1;
	  arm5_jump_table->jmp_b[dill_jmp_beqi] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_beqi].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_beqi].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bequ] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bequ].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_bequ].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bequl] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bequl].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_bequl].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_beql] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_beql].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_beql].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_beqp] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_beqp].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_beqp].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_beqd] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_beqd].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_beqd].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_beqf] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_beqf].data1 = dill_eq_code;
	  arm5_jump_table->b_data[dill_jmp_beqf].data2 = DILL_F;
	  arm5_jump_table->jmp_b[dill_jmp_bgei] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgei].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgei].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bgeu] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgeu].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgeu].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bgeul] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgeul].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgeul].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_bgel] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgel].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgel].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_bgep] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgep].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgep].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_bged] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bged].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bged].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_bgef] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgef].data1 = dill_ge_code;
	  arm5_jump_table->b_data[dill_jmp_bgef].data2 = DILL_F;
	  arm5_jump_table->jmp_b[dill_jmp_bgti] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgti].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgti].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bgtu] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtu].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtu].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bgtul] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtul].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtul].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_bgtl] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtl].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtl].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_bgtp] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtp].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtp].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_bgtd] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtd].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtd].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_bgtf] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bgtf].data1 = dill_gt_code;
	  arm5_jump_table->b_data[dill_jmp_bgtf].data2 = DILL_F;
	  arm5_jump_table->jmp_b[dill_jmp_blei] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_blei].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_blei].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bleu] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bleu].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_bleu].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bleul] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bleul].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_bleul].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_blel] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_blel].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_blel].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_blep] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_blep].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_blep].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_bled] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bled].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_bled].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_blef] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_blef].data1 = dill_le_code;
	  arm5_jump_table->b_data[dill_jmp_blef].data2 = DILL_F;
	  arm5_jump_table->jmp_b[dill_jmp_blti] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_blti].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_blti].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bltu] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltu].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltu].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bltul] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltul].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltul].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_bltl] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltl].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltl].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_bltp] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltp].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltp].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_bltd] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltd].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltd].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_bltf] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bltf].data1 = dill_lt_code;
	  arm5_jump_table->b_data[dill_jmp_bltf].data2 = DILL_F;
	  arm5_jump_table->jmp_b[dill_jmp_bnei] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bnei].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bnei].data2 = DILL_I;
	  arm5_jump_table->jmp_b[dill_jmp_bneu] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bneu].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bneu].data2 = DILL_U;
	  arm5_jump_table->jmp_b[dill_jmp_bneul] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bneul].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bneul].data2 = DILL_UL;
	  arm5_jump_table->jmp_b[dill_jmp_bnel] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bnel].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bnel].data2 = DILL_L;
	  arm5_jump_table->jmp_b[dill_jmp_bnep] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bnep].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bnep].data2 = DILL_P;
	  arm5_jump_table->jmp_b[dill_jmp_bned] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bned].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bned].data2 = DILL_D;
	  arm5_jump_table->jmp_b[dill_jmp_bnef] = arm5_branch;
	  arm5_jump_table->b_data[dill_jmp_bnef].data1 = dill_ne_code;
	  arm5_jump_table->b_data[dill_jmp_bnef].data2 = DILL_F;
	  arm5_jump_table->jmp_bi[dill_jmp_beqi] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bequ] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bequl] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_beql] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_beqp] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgei] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgeu] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgeul] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgel] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgep] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgti] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgtu] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgtul] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgtl] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bgtp] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_blei] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bleu] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bleul] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_blel] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_blep] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_blti] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bltu] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bltul] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bltl] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bltp] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bnei] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bneu] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bneul] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bnel] = arm5_branchi;
	  arm5_jump_table->jmp_bi[dill_jmp_bnep] = arm5_branchi;
	  arm5_jump_table->proc_start = (dill_mach_proc_start) arm5_proc_start;
	  arm5_jump_table->end = arm5_end;
	  arm5_jump_table->type_align = arm5_type_align;
	  arm5_jump_table->type_size = arm5_type_size;
	  arm5_jump_table->clone_code = arm5_clone_code;
	  arm5_jump_table->ret = arm5_ret;
	  arm5_jump_table->reti = arm5_reti;
	  arm5_jump_table->retf = (ret_opf)arm5_reti;
	  arm5_jump_table->load = arm5_pload;
	  arm5_jump_table->bsload = arm5_pbsload;
	  arm5_jump_table->loadi = arm5_ploadi;
	  arm5_jump_table->bsloadi = arm5_pbsloadi;
	  arm5_jump_table->loadi = arm5_ploadi;
	  arm5_jump_table->store = arm5_pstore;
	  arm5_jump_table->storei = arm5_pstorei;
	  arm5_jump_table->convert = arm5_convert;
	  arm5_jump_table->mov = arm5_mov;
	  arm5_jump_table->set = arm5_pset;
	  arm5_jump_table->setf = arm5_setf;
	  arm5_jump_table->jv = arm5_jump_to_label;
	  arm5_jump_table->jp = arm5_jump_to_reg;
	  arm5_jump_table->jpi = arm5_jump_to_imm;
	  arm5_jump_table->jal = arm5_jal;
	  arm5_jump_table->push = arm5_push;
	  arm5_jump_table->pushi = arm5_pushi;
	  arm5_jump_table->pushfi = arm5_pushfi;
	  arm5_jump_table->pushpi = arm5_pushpi;
	  arm5_jump_table->calli = arm5_calli;
	  arm5_jump_table->callr = arm5_callr;
	  arm5_jump_table->local = arm5_local_op;
	  arm5_jump_table->save_restore = arm5_save_restore_op;
	  arm5_jump_table->init_disassembly = arm5_init_disassembly_info;
	  arm5_jump_table->print_insn = arm5_print_insn;
	  arm5_jump_table->print_reg = arm5_print_reg;
	  arm5_jump_table->count_insn = arm5_count_insn;
	  arm5_jump_table->do_reverse_push = 0;
	  arm5_jump_table->target_byte_order = 2;  /* Format_Integer_littleendian */
	  arm5_jump_table->target_float_format = 3;  /* Format_IEEE_754_mixedendian */ 
}

extern void dill_arm5_init(dill_stream s)
{
	gen_arm5_jump_table();
	s->j = arm5_jump_table;
	s->dill_local_pointer = _fp;
	s->dill_param_reg_pointer = _fp;
	s->p->mach_info = (void*)gen_arm5_mach_info(s,/*v9*/ 0);
	s->p->machine_strr_tmp_reg = _v1;
	return;
}
extern void dill_armv5_init(dill_stream s)
{
	gen_arm5_jump_table();
	s->j = arm5_jump_table;
	s->dill_local_pointer = _fp;
	s->dill_param_reg_pointer = _fp;
	s->p->mach_info = (void*)gen_arm5_mach_info(s,/*v9*/ 1);
	s->p->machine_strr_tmp_reg = _v1;
	return;
}
