! SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
!
! SPDX-License-Identifier: Apache-2.0


module adios2

    use adios2_parameters_mod
    use adios2_adios_mod
    use adios2_attribute_mod
    use adios2_attribute_data_mod
    use adios2_io_mod
    use adios2_variable_mod
    use adios2_variable_min_mod
    use adios2_variable_max_mod
    use adios2_engine_mod

end module
