#!/usr/bin/env bash

set -e
set -x
shopt -s dotglob

readonly name="dill"
readonly ownership="dill Upstream <robot@adios2>"
readonly subtree="thirdparty/dill/dill"
readonly repo="https://github.com/GTkorvo/dill.git"
readonly tag="master"
readonly shortlog="true"
readonly exact_tree_match="false"
readonly paths="
"

extract_source () {
    git_archive
    pushd "$extractdir/$name-reduced"
    rm -rf ./scripts/
    rm -rf .github/
    popd
}

. "${BASH_SOURCE%/*}/../update-common.sh"
