/*
 * SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef ADIOS2_BINDINGS_FORTRAN_F2C_ADIOS2_F2C_COMMON_H_
#define ADIOS2_BINDINGS_FORTRAN_F2C_ADIOS2_F2C_COMMON_H_

#include "adios2/common/ADIOSConfig.h"
#include <FC.h>
#include <adios2_c.h>

#include <stdint.h> // int64_t
#include <vector>

#endif /* ADIOS2_BINDINGS_FORTRAN_F2C_ADIOS2_F2C_COMMON_H_ */
