#!/bin/bash

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

set +u

export CONDA_DLL_SEARCH_MODIFICATION_ENABLE=1
conda activate adios2
