/* This file is generated from ia64.ops.  Do not edit directly. */

#include "config.h"
#include "dill.h"
#include "dill_internal.h"
#include "ia64.h"
DECLARE_JUMP_TABLE(ia64);
extern void dill_ia64_init(dill_stream c)
{
	  FILL_JUMP_STRUCTURE(ia64);
	  ia64_jump_table->jmp_a3[dill_jmp_addi] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_addi].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_addi].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_addu] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_addu].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_addu].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_addul] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_addul].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_addul].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_addl] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_addl].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_addl].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_addp] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_addp].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_addp].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_subi] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_subi].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_subi].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_subu] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_subu].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_subu].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_subul] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_subul].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_subul].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_subl] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_subl].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_subl].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_subp] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_subp].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_subp].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_mulu] = ia64_mul;
	  ia64_jump_table->a3_data[dill_jmp_mulu].data1 = 0 /*unsigned*/;
	  ia64_jump_table->a3_data[dill_jmp_mulu].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_mulul] = ia64_mul;
	  ia64_jump_table->a3_data[dill_jmp_mulul].data1 = 0 /*unsigned*/;
	  ia64_jump_table->a3_data[dill_jmp_mulul].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_muli] = ia64_mul;
	  ia64_jump_table->a3_data[dill_jmp_muli].data1 = 1 /*signed*/;
	  ia64_jump_table->a3_data[dill_jmp_muli].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_mull] = ia64_mul;
	  ia64_jump_table->a3_data[dill_jmp_mull].data1 = 1 /*signed*/;
	  ia64_jump_table->a3_data[dill_jmp_mull].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_divi] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_divi].data1 = DILL_I;
	  ia64_jump_table->a3_data[dill_jmp_divi].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_divu] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_divu].data1 = DILL_U;
	  ia64_jump_table->a3_data[dill_jmp_divu].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_divl] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_divl].data1 = DILL_L;
	  ia64_jump_table->a3_data[dill_jmp_divl].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_divul] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_divul].data1 = DILL_UL;
	  ia64_jump_table->a3_data[dill_jmp_divul].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_modi] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_modi].data1 = DILL_I;
	  ia64_jump_table->a3_data[dill_jmp_modi].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_modu] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_modu].data1 = DILL_U;
	  ia64_jump_table->a3_data[dill_jmp_modu].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_modl] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_modl].data1 = DILL_L;
	  ia64_jump_table->a3_data[dill_jmp_modl].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_modul] = ia64_div_mod;
	  ia64_jump_table->a3_data[dill_jmp_modul].data1 = DILL_UL;
	  ia64_jump_table->a3_data[dill_jmp_modul].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_andi] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_andi].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_andi].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_andu] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_andu].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_andu].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_andul] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_andul].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_andul].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_andl] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_andl].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_andl].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_ori] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_ori].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_ori].data2 = 2;
	  ia64_jump_table->jmp_a3[dill_jmp_oru] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_oru].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_oru].data2 = 2;
	  ia64_jump_table->jmp_a3[dill_jmp_orul] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_orul].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_orul].data2 = 2;
	  ia64_jump_table->jmp_a3[dill_jmp_orl] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_orl].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_orl].data2 = 2;
	  ia64_jump_table->jmp_a3[dill_jmp_xori] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_xori].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_xori].data2 = 3;
	  ia64_jump_table->jmp_a3[dill_jmp_xoru] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_xoru].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_xoru].data2 = 3;
	  ia64_jump_table->jmp_a3[dill_jmp_xorul] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_xorul].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_xorul].data2 = 3;
	  ia64_jump_table->jmp_a3[dill_jmp_xorl] = ia64_arith3;
	  ia64_jump_table->a3_data[dill_jmp_xorl].data1 = 3;
	  ia64_jump_table->a3_data[dill_jmp_xorl].data2 = 3;
	  ia64_jump_table->jmp_a3[dill_jmp_lshi] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_lshi].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_lshi].data2 = DILL_I;
	  ia64_jump_table->jmp_a3[dill_jmp_lshu] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_lshu].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_lshu].data2 = DILL_U;
	  ia64_jump_table->jmp_a3[dill_jmp_lshul] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_lshul].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_lshul].data2 = DILL_UL;
	  ia64_jump_table->jmp_a3[dill_jmp_lshl] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_lshl].data1 = 1;
	  ia64_jump_table->a3_data[dill_jmp_lshl].data2 = DILL_L;
	  ia64_jump_table->jmp_a3[dill_jmp_rshi] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_rshi].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_rshi].data2 = DILL_I;
	  ia64_jump_table->jmp_a3[dill_jmp_rshu] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_rshu].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_rshu].data2 = DILL_U;
	  ia64_jump_table->jmp_a3[dill_jmp_rshl] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_rshl].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_rshl].data2 = DILL_L;
	  ia64_jump_table->jmp_a3[dill_jmp_rshul] = ia64_shift;
	  ia64_jump_table->a3_data[dill_jmp_rshul].data1 = 0;
	  ia64_jump_table->a3_data[dill_jmp_rshul].data2 = DILL_UL;
	  ia64_jump_table->jmp_a3[dill_jmp_addf] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_addf].data1 = 0x8;
	  ia64_jump_table->a3_data[dill_jmp_addf].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_addd] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_addd].data1 = 0x9;
	  ia64_jump_table->a3_data[dill_jmp_addd].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_subf] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_subf].data1 = 0xa;
	  ia64_jump_table->a3_data[dill_jmp_subf].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_subd] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_subd].data1 = 0xb;
	  ia64_jump_table->a3_data[dill_jmp_subd].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_mulf] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_mulf].data1 = 0x18;
	  ia64_jump_table->a3_data[dill_jmp_mulf].data2 = 1;
	  ia64_jump_table->jmp_a3[dill_jmp_muld] = ia64_farith;
	  ia64_jump_table->a3_data[dill_jmp_muld].data1 = 0x19;
	  ia64_jump_table->a3_data[dill_jmp_muld].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_divf] = ia64_fdiv;
	  ia64_jump_table->a3_data[dill_jmp_divf].data1 = DILL_F;
	  ia64_jump_table->a3_data[dill_jmp_divf].data2 = 0;
	  ia64_jump_table->jmp_a3[dill_jmp_divd] = ia64_fdiv;
	  ia64_jump_table->a3_data[dill_jmp_divd].data1 = DILL_D;
	  ia64_jump_table->a3_data[dill_jmp_divd].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_addi] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_addi].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_addi].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_addu] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_addu].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_addu].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_addul] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_addul].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_addul].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_addl] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_addl].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_addl].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_addp] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_addp].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_addp].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_subi] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_subi].data1 = 9;
	  ia64_jump_table->a3i_data[dill_jmp_subi].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_subu] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_subu].data1 = 9;
	  ia64_jump_table->a3i_data[dill_jmp_subu].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_subul] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_subul].data1 = 9;
	  ia64_jump_table->a3i_data[dill_jmp_subul].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_subl] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_subl].data1 = 9;
	  ia64_jump_table->a3i_data[dill_jmp_subl].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_subp] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_subp].data1 = 9;
	  ia64_jump_table->a3i_data[dill_jmp_subp].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_mulu] = (arith_op3i)ia64_muli;
	  ia64_jump_table->a3i_data[dill_jmp_mulu].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_mulu].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_mulul] = (arith_op3i)ia64_muli;
	  ia64_jump_table->a3i_data[dill_jmp_mulul].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_mulul].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_muli] = (arith_op3i)ia64_muli;
	  ia64_jump_table->a3i_data[dill_jmp_muli].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_muli].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_mull] = (arith_op3i)ia64_muli;
	  ia64_jump_table->a3i_data[dill_jmp_mull].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_mull].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_modi] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_modi].data1 = DILL_I;
	  ia64_jump_table->a3i_data[dill_jmp_modi].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_modu] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_modu].data1 = DILL_U;
	  ia64_jump_table->a3i_data[dill_jmp_modu].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_modl] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_modl].data1 = DILL_L;
	  ia64_jump_table->a3i_data[dill_jmp_modl].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_modul] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_modul].data1 = DILL_UL;
	  ia64_jump_table->a3i_data[dill_jmp_modul].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_divi] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_divi].data1 = DILL_I;
	  ia64_jump_table->a3i_data[dill_jmp_divi].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_divu] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_divu].data1 = DILL_U;
	  ia64_jump_table->a3i_data[dill_jmp_divu].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_divl] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_divl].data1 = DILL_L;
	  ia64_jump_table->a3i_data[dill_jmp_divl].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_divul] = ia64_div_modi;
	  ia64_jump_table->a3i_data[dill_jmp_divul].data1 = DILL_UL;
	  ia64_jump_table->a3i_data[dill_jmp_divul].data2 = 1;
	  ia64_jump_table->jmp_a3i[dill_jmp_andi] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_andi].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_andi].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_andu] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_andu].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_andu].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_andul] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_andul].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_andul].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_andl] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_andl].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_andl].data2 = 0;
	  ia64_jump_table->jmp_a3i[dill_jmp_ori] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_ori].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_ori].data2 = 2;
	  ia64_jump_table->jmp_a3i[dill_jmp_oru] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_oru].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_oru].data2 = 2;
	  ia64_jump_table->jmp_a3i[dill_jmp_orul] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_orul].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_orul].data2 = 2;
	  ia64_jump_table->jmp_a3i[dill_jmp_orl] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_orl].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_orl].data2 = 2;
	  ia64_jump_table->jmp_a3i[dill_jmp_xori] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_xori].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_xori].data2 = 3;
	  ia64_jump_table->jmp_a3i[dill_jmp_xoru] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_xoru].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_xoru].data2 = 3;
	  ia64_jump_table->jmp_a3i[dill_jmp_xorul] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_xorul].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_xorul].data2 = 3;
	  ia64_jump_table->jmp_a3i[dill_jmp_xorl] = ia64_arith3i;
	  ia64_jump_table->a3i_data[dill_jmp_xorl].data1 = 0xb;
	  ia64_jump_table->a3i_data[dill_jmp_xorl].data2 = 3;
	  ia64_jump_table->jmp_a3i[dill_jmp_lshi] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_lshi].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_lshi].data2 = DILL_I;
	  ia64_jump_table->jmp_a3i[dill_jmp_lshu] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_lshu].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_lshu].data2 = DILL_U;
	  ia64_jump_table->jmp_a3i[dill_jmp_lshul] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_lshul].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_lshul].data2 = DILL_UL;
	  ia64_jump_table->jmp_a3i[dill_jmp_lshl] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_lshl].data1 = 1;
	  ia64_jump_table->a3i_data[dill_jmp_lshl].data2 = DILL_L;
	  ia64_jump_table->jmp_a3i[dill_jmp_rshi] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_rshi].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_rshi].data2 = DILL_I;
	  ia64_jump_table->jmp_a3i[dill_jmp_rshu] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_rshu].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_rshu].data2 = DILL_U;
	  ia64_jump_table->jmp_a3i[dill_jmp_rshul] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_rshul].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_rshul].data2 = DILL_UL;
	  ia64_jump_table->jmp_a3i[dill_jmp_rshl] = ia64_shifti;
	  ia64_jump_table->a3i_data[dill_jmp_rshl].data1 = 0;
	  ia64_jump_table->a3i_data[dill_jmp_rshl].data2 = DILL_L;
	  ia64_jump_table->jmp_a2[dill_jmp_comi] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_comi].data1 = 1;
	  ia64_jump_table->a2_data[dill_jmp_comi].data2 = 1;
	  ia64_jump_table->jmp_a2[dill_jmp_comu] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_comu].data1 = 1;
	  ia64_jump_table->a2_data[dill_jmp_comu].data2 = 1;
	  ia64_jump_table->jmp_a2[dill_jmp_comul] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_comul].data1 = 1;
	  ia64_jump_table->a2_data[dill_jmp_comul].data2 = 1;
	  ia64_jump_table->jmp_a2[dill_jmp_coml] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_coml].data1 = 1;
	  ia64_jump_table->a2_data[dill_jmp_coml].data2 = 1;
	  ia64_jump_table->jmp_a2[dill_jmp_negi] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_negi].data1 = 2;
	  ia64_jump_table->a2_data[dill_jmp_negi].data2 = 2;
	  ia64_jump_table->jmp_a2[dill_jmp_negu] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_negu].data1 = 2;
	  ia64_jump_table->a2_data[dill_jmp_negu].data2 = 2;
	  ia64_jump_table->jmp_a2[dill_jmp_negul] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_negul].data1 = 2;
	  ia64_jump_table->a2_data[dill_jmp_negul].data2 = 2;
	  ia64_jump_table->jmp_a2[dill_jmp_negl] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_negl].data1 = 2;
	  ia64_jump_table->a2_data[dill_jmp_negl].data2 = 2;
	  ia64_jump_table->jmp_a2[dill_jmp_noti] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_noti].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_noti].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_notu] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_notu].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_notu].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_notul] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_notul].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_notul].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_notl] = ia64_arith2;
	  ia64_jump_table->a2_data[dill_jmp_notl].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_notl].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_negf] = ia64_farith2;
	  ia64_jump_table->a2_data[dill_jmp_negf].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_negf].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_negd] = ia64_farith2;
	  ia64_jump_table->a2_data[dill_jmp_negd].data1 = 0;
	  ia64_jump_table->a2_data[dill_jmp_negd].data2 = 0;
	  ia64_jump_table->jmp_a2[dill_jmp_bswaps] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswaps].data1 = DILL_S;
	  ia64_jump_table->a2_data[dill_jmp_bswaps].data2 = DILL_S;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapus] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapus].data1 = DILL_US;
	  ia64_jump_table->a2_data[dill_jmp_bswapus].data2 = DILL_US;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapi] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapi].data1 = DILL_I;
	  ia64_jump_table->a2_data[dill_jmp_bswapi].data2 = DILL_I;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapu] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapu].data1 = DILL_U;
	  ia64_jump_table->a2_data[dill_jmp_bswapu].data2 = DILL_U;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapul] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapul].data1 = DILL_UL;
	  ia64_jump_table->a2_data[dill_jmp_bswapul].data2 = DILL_UL;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapl] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapl].data1 = DILL_L;
	  ia64_jump_table->a2_data[dill_jmp_bswapl].data2 = DILL_L;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapf] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapf].data1 = DILL_F;
	  ia64_jump_table->a2_data[dill_jmp_bswapf].data2 = DILL_F;
	  ia64_jump_table->jmp_a2[dill_jmp_bswapd] = ia64_bswap;
	  ia64_jump_table->a2_data[dill_jmp_bswapd].data1 = DILL_D;
	  ia64_jump_table->a2_data[dill_jmp_bswapd].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_beqc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqc].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqc].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bequc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bequc].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_bequc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_beqs] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqs].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqs].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bequs] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bequs].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_bequs].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_beqi] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqi].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqi].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bequ] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bequ].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_bequ].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bequl] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bequl].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_bequl].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_beql] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beql].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beql].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_beqp] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqp].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqp].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_beqd] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqd].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqd].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_beqf] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_beqf].data1 = dill_eq_code;
	  ia64_jump_table->b_data[dill_jmp_beqf].data2 = DILL_F;
	  ia64_jump_table->jmp_b[dill_jmp_bgec] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgec].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgec].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bgeuc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgeuc].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgeuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_bges] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bges].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bges].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bgeus] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgeus].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgeus].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_bgei] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgei].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgei].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bgeu] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgeu].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgeu].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bgeul] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgeul].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgeul].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_bgel] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgel].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgel].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_bgep] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgep].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgep].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_bged] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bged].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bged].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_bgef] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgef].data1 = dill_ge_code;
	  ia64_jump_table->b_data[dill_jmp_bgef].data2 = DILL_F;
	  ia64_jump_table->jmp_b[dill_jmp_bgtc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtc].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtc].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bgtuc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtuc].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_bgts] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgts].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgts].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bgtus] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtus].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtus].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_bgti] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgti].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgti].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bgtu] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtu].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtu].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bgtul] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtul].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtul].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_bgtl] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtl].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtl].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_bgtp] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtp].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtp].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_bgtd] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtd].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtd].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_bgtf] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bgtf].data1 = dill_gt_code;
	  ia64_jump_table->b_data[dill_jmp_bgtf].data2 = DILL_F;
	  ia64_jump_table->jmp_b[dill_jmp_blec] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blec].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_blec].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bleuc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bleuc].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bleuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_bles] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bles].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bles].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bleus] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bleus].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bleus].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_blei] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blei].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_blei].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bleu] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bleu].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bleu].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bleul] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bleul].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bleul].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_blel] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blel].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_blel].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_blep] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blep].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_blep].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_bled] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bled].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_bled].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_blef] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blef].data1 = dill_le_code;
	  ia64_jump_table->b_data[dill_jmp_blef].data2 = DILL_F;
	  ia64_jump_table->jmp_b[dill_jmp_bltc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltc].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltc].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bltuc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltuc].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_blts] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blts].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_blts].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bltus] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltus].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltus].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_blti] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_blti].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_blti].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bltu] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltu].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltu].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bltul] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltul].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltul].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_bltl] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltl].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltl].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_bltp] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltp].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltp].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_bltd] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltd].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltd].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_bltf] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bltf].data1 = dill_lt_code;
	  ia64_jump_table->b_data[dill_jmp_bltf].data2 = DILL_F;
	  ia64_jump_table->jmp_b[dill_jmp_bnec] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnec].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnec].data2 = DILL_C;
	  ia64_jump_table->jmp_b[dill_jmp_bneuc] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bneuc].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bneuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_b[dill_jmp_bnes] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnes].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnes].data2 = DILL_S;
	  ia64_jump_table->jmp_b[dill_jmp_bneus] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bneus].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bneus].data2 = DILL_US;
	  ia64_jump_table->jmp_b[dill_jmp_bnei] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnei].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnei].data2 = DILL_I;
	  ia64_jump_table->jmp_b[dill_jmp_bneu] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bneu].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bneu].data2 = DILL_U;
	  ia64_jump_table->jmp_b[dill_jmp_bneul] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bneul].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bneul].data2 = DILL_UL;
	  ia64_jump_table->jmp_b[dill_jmp_bnel] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnel].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnel].data2 = DILL_L;
	  ia64_jump_table->jmp_b[dill_jmp_bnep] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnep].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnep].data2 = DILL_P;
	  ia64_jump_table->jmp_b[dill_jmp_bned] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bned].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bned].data2 = DILL_D;
	  ia64_jump_table->jmp_b[dill_jmp_bnef] = ia64_branch;
	  ia64_jump_table->b_data[dill_jmp_bnef].data1 = dill_ne_code;
	  ia64_jump_table->b_data[dill_jmp_bnef].data2 = DILL_F;
	  ia64_jump_table->jmp_bi[dill_jmp_beqc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bequc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_beqs] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bequs] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_beqi] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bequ] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bequl] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_beql] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_beqp] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgec] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgeuc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bges] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgeus] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgei] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgeu] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgeul] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgel] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgep] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtuc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgts] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtus] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgti] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtu] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtul] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtl] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bgtp] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blec] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bleuc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bles] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bleus] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blei] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bleu] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bleul] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blel] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blep] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltuc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blts] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltus] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_blti] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltu] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltul] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltl] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bltp] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bnec] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bneuc] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bnes] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bneus] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bnei] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bneu] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bneul] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bnel] = ia64_branchi;
	  ia64_jump_table->jmp_bi[dill_jmp_bnep] = ia64_branchi;
	  ia64_jump_table->jmp_c[dill_jmp_ceqc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqc].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqc].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cequc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cequc].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_cequc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_ceqs] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqs].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqs].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cequs] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cequs].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_cequs].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_ceqi] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqi].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqi].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cequ] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cequ].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_cequ].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cequl] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cequl].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_cequl].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_ceql] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceql].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceql].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_ceqp] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqp].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqp].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_ceqd] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqd].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqd].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_ceqf] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_ceqf].data1 = dill_eq_code;
	  ia64_jump_table->c_data[dill_jmp_ceqf].data2 = DILL_F;
	  ia64_jump_table->jmp_c[dill_jmp_cgec] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgec].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgec].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cgeuc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgeuc].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgeuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_cges] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cges].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cges].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cgeus] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgeus].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgeus].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_cgei] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgei].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgei].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cgeu] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgeu].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgeu].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cgeul] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgeul].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgeul].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_cgel] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgel].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgel].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_cgep] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgep].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgep].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_cged] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cged].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cged].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_cgef] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgef].data1 = dill_ge_code;
	  ia64_jump_table->c_data[dill_jmp_cgef].data2 = DILL_F;
	  ia64_jump_table->jmp_c[dill_jmp_cgtc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtc].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtc].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cgtuc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtuc].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_cgts] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgts].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgts].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cgtus] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtus].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtus].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_cgti] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgti].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgti].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cgtu] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtu].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtu].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cgtul] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtul].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtul].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_cgtl] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtl].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtl].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_cgtp] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtp].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtp].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_cgtd] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtd].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtd].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_cgtf] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cgtf].data1 = dill_gt_code;
	  ia64_jump_table->c_data[dill_jmp_cgtf].data2 = DILL_F;
	  ia64_jump_table->jmp_c[dill_jmp_clec] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clec].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_clec].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cleuc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cleuc].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cleuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_cles] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cles].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cles].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cleus] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cleus].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cleus].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_clei] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clei].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_clei].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cleu] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cleu].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cleu].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cleul] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cleul].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cleul].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_clel] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clel].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_clel].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_clep] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clep].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_clep].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_cled] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cled].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_cled].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_clef] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clef].data1 = dill_le_code;
	  ia64_jump_table->c_data[dill_jmp_clef].data2 = DILL_F;
	  ia64_jump_table->jmp_c[dill_jmp_cltc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltc].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltc].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cltuc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltuc].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_clts] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clts].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_clts].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cltus] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltus].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltus].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_clti] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_clti].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_clti].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cltu] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltu].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltu].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cltul] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltul].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltul].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_cltl] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltl].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltl].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_cltp] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltp].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltp].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_cltd] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltd].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltd].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_cltf] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cltf].data1 = dill_lt_code;
	  ia64_jump_table->c_data[dill_jmp_cltf].data2 = DILL_F;
	  ia64_jump_table->jmp_c[dill_jmp_cnec] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnec].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnec].data2 = DILL_C;
	  ia64_jump_table->jmp_c[dill_jmp_cneuc] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cneuc].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cneuc].data2 = DILL_UC;
	  ia64_jump_table->jmp_c[dill_jmp_cnes] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnes].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnes].data2 = DILL_S;
	  ia64_jump_table->jmp_c[dill_jmp_cneus] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cneus].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cneus].data2 = DILL_US;
	  ia64_jump_table->jmp_c[dill_jmp_cnei] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnei].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnei].data2 = DILL_I;
	  ia64_jump_table->jmp_c[dill_jmp_cneu] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cneu].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cneu].data2 = DILL_U;
	  ia64_jump_table->jmp_c[dill_jmp_cneul] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cneul].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cneul].data2 = DILL_UL;
	  ia64_jump_table->jmp_c[dill_jmp_cnel] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnel].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnel].data2 = DILL_L;
	  ia64_jump_table->jmp_c[dill_jmp_cnep] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnep].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnep].data2 = DILL_P;
	  ia64_jump_table->jmp_c[dill_jmp_cned] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cned].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cned].data2 = DILL_D;
	  ia64_jump_table->jmp_c[dill_jmp_cnef] = ia64_compare;
	  ia64_jump_table->c_data[dill_jmp_cnef].data1 = dill_ne_code;
	  ia64_jump_table->c_data[dill_jmp_cnef].data2 = DILL_F;
	  ia64_jump_table->jmp_ci[dill_jmp_ceqc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cequc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_ceqs] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cequs] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_ceqi] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cequ] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cequl] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_ceql] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_ceqp] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgec] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgeuc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cges] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgeus] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgei] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgeu] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgeul] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgel] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgep] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtuc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgts] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtus] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgti] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtu] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtul] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtl] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cgtp] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clec] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cleuc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cles] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cleus] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clei] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cleu] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cleul] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clel] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clep] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltuc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clts] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltus] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_clti] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltu] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltul] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltl] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cltp] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cnec] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cneuc] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cnes] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cneus] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cnei] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cneu] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cneul] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cnel] = ia64_comparei;
	  ia64_jump_table->jmp_ci[dill_jmp_cnep] = ia64_comparei;
	  ia64_jump_table->proc_start = (dill_mach_proc_start) ia64_proc_start;
	  ia64_jump_table->end = ia64_end;
	  ia64_jump_table->package_end = ia64_package_end;
	  ia64_jump_table->clone_code = ia64_clone_code;
	  ia64_jump_table->type_align = ia64_type_align;
	  ia64_jump_table->type_size = ia64_type_size;
	  ia64_jump_table->ret = ia64_ret;
	  ia64_jump_table->reti = ia64_reti;
	  ia64_jump_table->retf = ia64_retf;
	  ia64_jump_table->load = ia64_pload;
	  ia64_jump_table->bsload = ia64_pbsload;
	  ia64_jump_table->loadi = ia64_ploadi;
	  ia64_jump_table->bsloadi = ia64_pbsloadi;
	  ia64_jump_table->loadi = ia64_ploadi;
	  ia64_jump_table->store = ia64_pstore;
	  ia64_jump_table->storei = ia64_pstorei;
	  ia64_jump_table->convert = ia64_convert;
	  ia64_jump_table->mov = ia64_mov;
	  ia64_jump_table->set = ia64_pset;
	  ia64_jump_table->setf = ia64_setf;
	  ia64_jump_table->setp = ia64_setp;
	  ia64_jump_table->jv = ia64_jump_to_label;
	  ia64_jump_table->jp = ia64_jump_to_reg;
	  ia64_jump_table->jpi = ia64_jump_to_imm;
	  ia64_jump_table->jal = ia64_jal;
	  ia64_jump_table->push = ia64_push;
	  ia64_jump_table->pushi = ia64_pushi;
	  ia64_jump_table->pushpi = ia64_pushpi;
	  ia64_jump_table->pushfi = ia64_pushfi;
	  ia64_jump_table->calli = ia64_calli;
	  ia64_jump_table->callr = ia64_callr;
	  ia64_jump_table->local = ia64_local_op;
	  ia64_jump_table->save_restore = ia64_save_restore_op;
	  ia64_jump_table->init_disassembly = ia64_init_disassembly_info;
	  ia64_jump_table->print_insn = ia64_print_insn;
	  ia64_jump_table->print_reg = ia64_print_reg;
	  ia64_jump_table->count_insn = ia64_count_insn;
	  ia64_jump_table->do_reverse_push = 0;
	  ia64_jump_table->target_byte_order = 2;  /* Format_Integer_littleendian */
	  ia64_jump_table->target_float_format = 2;  /* Format_IEEE_754_littleendian */ 
	c->j = ia64_jump_table;
	c->dill_local_pointer = L0;
	c->dill_param_reg_pointer = L0;
	c->p->mach_info = (void*)gen_ia64_mach_info(c);
	c->p->machine_strr_tmp_reg = R8;
	return;
}
