/*
 *  This file is automatically generated by gen_interface.pl from evpath.h.
 *
 *  DO NOT EDIT
 *
 */

#ifdef  __cplusplus
extern "C" {
#endif

extern int
REVaction_add_split_target(CMConnection conn, EVstone stone, EVaction action, EVstone target_stone);

extern void
REVaction_remove_split_target(CMConnection conn, EVstone stone, EVaction action, EVstone target_stone);

extern int
REVaction_set_output(CMConnection conn, EVstone stone, EVaction action, int output_index, EVstone target_stone);

extern EVstone
REValloc_stone(CMConnection conn);

extern EVaction
REVassoc_bridge_action(CMConnection conn, EVstone stone, attr_list contact_list, EVstone remote_stone);

extern EVaction
REVassoc_filter_action(CMConnection conn, EVstone stone, FMStructDescList incoming_format_list, char *handler, EVstone out_stone);

extern EVaction
REVassoc_immediate_action(CMConnection conn, EVstone stone, char *action_spec);

extern EVaction
REVassoc_multi_action(CMConnection conn, EVstone stone, char *action_spec);

extern EVaction
REVassoc_split_action(CMConnection conn, EVstone stone, EVstone *target_list);

extern EVaction
REVassoc_store_action(CMConnection conn, EVstone stone_num, EVstone out_stone, int store_limit);

extern EVaction
REVassoc_terminal_action(CMConnection conn, EVstone stone, FMStructDescList format_list, char *handler);

extern void
REVclear_stored(CMConnection conn, EVstone stone_num, EVaction action_num);

extern EVstone
REVcreate_auto_stone(CMConnection conn, int period_sec, int period_usec, char *action_spec, EVstone out_stone);

extern EVstone
REVcreate_bridge_action(CMConnection conn, attr_list contact_list, EVstone remote_stone);

extern EVstone
REVcreate_immediate_action(CMConnection conn, char *action_spec, EVstone *target_list);

extern EVstone
REVcreate_split_action(CMConnection conn, EVstone *target_list);

extern EVstone
REVcreate_stone_action(CMConnection conn, char *action_spec);

extern EVstone
REVcreate_store_action(CMConnection conn, EVstone out_stone, int store_limit);

extern EVstone
REVcreate_terminal_action(CMConnection conn, FMStructDescList format_list, char *handler);

extern int
REVdestroy_stone(CMConnection conn, EVstone stone_id);

extern int
REVdrain_stone(CMConnection conn, EVstone stone_id);

extern void
REVenable_auto_stone(CMConnection conn, EVstone stone_num, int period_sec, int period_usec);

extern attr_list
REVextract_attr_list(CMConnection conn, EVstone stone_id);

extern void
REVfree_stone(CMConnection conn, EVstone stone);

extern int
REVfreeze_stone(CMConnection conn, EVstone stone_id);

extern void
REVset_attr_list(CMConnection conn, EVstone stone_id, attr_list list);

extern void
REVstall_stone(CMConnection conn, EVstone stone_id);

extern void
REVstone_add_split_target(CMConnection conn, EVstone stone, EVstone target_stone);

extern void
REVstone_remove_split_target(CMConnection conn, EVstone stone, EVstone target_stone);

extern int
REVstone_set_output(CMConnection conn, EVstone stone, int output_index, EVstone target_stone);

extern int
REVtransfer_events(CMConnection conn, EVstone src_stone, EVstone dest_stone);

extern int
REVunfreeze_stone(CMConnection conn, EVstone stone_id);

extern void
REVunstall_stone(CMConnection conn, EVstone stone_id);

#ifdef  __cplusplus
}
#endif
