/* This file is generated from sparc.ops.  Do not edit directly. */

#include "config.h"
#include "dill.h"
#include "dill_internal.h"
#include "sparc.h"
#include <string.h>
extern void sparc_FORM3_farith(s, op3, fop, dest, src1, src2)
dill_stream s;
int op3;
int fop;
int dest;
int src1;
int src2;
{
    /* format 3 */
    INSN_OUT(s, HDR(0x2)|OP3(op3)|RD(dest)|RS1(src1)|OPF(fop)|RS2(src2));
}

extern void sparc_FORM2_farith(s, op3, fop, dest, src)
dill_stream s;
int op3;
int fop;
int dest;
int src;
{
    /* format 3 */
    INSN_OUT(s, HDR(0x2)|OP3(op3)|RD(dest)|OPF(fop)|RS2(src));
}

extern void sparc_FORM2_arith(s, op3, op, dest, src)
dill_stream s;
int op3;
int op;
int dest;
int src;
{
    if (op != 0) {
	/* format 3 with g0 as rs1*/
	INSN_OUT(s, HDR(op)|OP3(op3)|RD(dest)|RS1(_g0)|RS2(src));
    } else {
	/* must be NOT */
	INSN_OUT(s, HDR(0x2)|RD(_g0)|OP3(0x14)|RS1(_g0)|RS2(src));/* subcc */
	INSN_OUT(s, HDR(0x2)|RD(dest)|OP3(0xc)|RS1(_g0)|IM|SIMM13(-1));
    }
}

static struct jmp_table_s sparc_jump_table_struct;
static arith_op3 sparc_arith_op3_table[dill_jmp_a3_size + 1];
static jmp_data  sparc_arith_op3_data[dill_jmp_a3_size + 1];
static arith_op3i sparc_arith_op3i_table[dill_jmp_a3_size + 1];
static jmp_data  sparc_arith_op3i_data[dill_jmp_a3_size + 1];
static arith_op2 sparc_arith_op2_table[dill_jmp_a2_size + 1];
static jmp_data  sparc_arith_op2_data[dill_jmp_a2_size + 1];
static branch_op sparc_branch_table[dill_jmp_branch_size + 1];
static branch_opi sparc_branchi_table[dill_jmp_branch_size + 1];
static jmp_data  sparc_branch_data[dill_jmp_branch_size + 1];
static compare_op sparc_compare_table[dill_jmp_compare_size + 1];
static compare_opi sparc_comparei_table[dill_jmp_compare_size + 1];
static jmp_data sparc_compare_data[dill_jmp_compare_size + 1];

jmp_table sparc_jump_table;
static void gen_sparc_jump_table()
{
    sparc_jump_table = &sparc_jump_table_struct;          
    memset(sparc_jump_table, 0, sizeof(struct jmp_table_s));
    /* arith3 */
    sparc_jump_table->jmp_a3 = sparc_arith_op3_table;
    memset(sparc_jump_table->jmp_a3, 0, sizeof(sparc_arith_op3_table));
    sparc_jump_table->a3_data = sparc_arith_op3_data;
    memset(sparc_jump_table->a3_data, 0, sizeof(sparc_arith_op3_data));
    /* arith3i */
    sparc_jump_table->jmp_a3i = sparc_arith_op3i_table;
    memset(sparc_jump_table->jmp_a3i, 0, sizeof(sparc_arith_op3i_table));
    sparc_jump_table->a3i_data = sparc_arith_op3i_data;
    memset(sparc_jump_table->a3i_data, 0, sizeof(sparc_arith_op3i_data));
    /* arith2 */
    sparc_jump_table->jmp_a2 = sparc_arith_op2_table;
    memset(sparc_jump_table->jmp_a2, 0, sizeof(sparc_arith_op2_table));
    sparc_jump_table->a2_data = sparc_arith_op2_data;
    memset(sparc_jump_table->a2_data, 0, sizeof(sparc_arith_op2_data));
    /* branch */
    sparc_jump_table->jmp_b = sparc_branch_table;
    memset(sparc_jump_table->jmp_b, 0, sizeof(sparc_branch_table));
    /* branchi */
    sparc_jump_table->jmp_bi = sparc_branchi_table;
    memset(sparc_jump_table->jmp_bi, 0, sizeof(sparc_branch_table));
    sparc_jump_table->b_data = sparc_branch_data;
    memset(sparc_jump_table->b_data, 0, sizeof(sparc_branch_data));
    /* compare */
    sparc_jump_table->jmp_c = sparc_compare_table;
    memset(sparc_jump_table->jmp_c, 0, sizeof(sparc_compare_table));
    /* comparei */
    sparc_jump_table->jmp_ci = sparc_comparei_table;
    memset(sparc_jump_table->jmp_ci, 0, sizeof(sparc_compare_table));
    sparc_jump_table->c_data = sparc_compare_data;
    memset(sparc_jump_table->c_data, 0, sizeof(sparc_compare_data));
    sparc_jump_table->jmp_a3[dill_jmp_addi] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_addi].data1 = 0x0;
    sparc_jump_table->a3_data[dill_jmp_addi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_addu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_addu].data1 = 0x0;
    sparc_jump_table->a3_data[dill_jmp_addu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_addul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_addul].data1 = 0x0;
    sparc_jump_table->a3_data[dill_jmp_addul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_addl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_addl].data1 = 0x0;
    sparc_jump_table->a3_data[dill_jmp_addl].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_addp] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_addp].data1 = 0x0;
    sparc_jump_table->a3_data[dill_jmp_addp].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_subi] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_subi].data1 = 0x4;
    sparc_jump_table->a3_data[dill_jmp_subi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_subu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_subu].data1 = 0x4;
    sparc_jump_table->a3_data[dill_jmp_subu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_subul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_subul].data1 = 0x4;
    sparc_jump_table->a3_data[dill_jmp_subul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_subl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_subl].data1 = 0x4;
    sparc_jump_table->a3_data[dill_jmp_subl].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_subp] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_subp].data1 = 0x4;
    sparc_jump_table->a3_data[dill_jmp_subp].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_mulu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_mulu].data1 = 0x9/*umul*/;
    sparc_jump_table->a3_data[dill_jmp_mulu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_mulul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_mulul].data1 = 0x9/*umul*/;
    sparc_jump_table->a3_data[dill_jmp_mulul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_muli] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_muli].data1 = 0x9/*smul*/;
    sparc_jump_table->a3_data[dill_jmp_muli].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_mull] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_mull].data1 = 0x9/*smul*/;
    sparc_jump_table->a3_data[dill_jmp_mull].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_divu] = sparc_div;
    sparc_jump_table->a3_data[dill_jmp_divu].data1 = 0x0d/*udiv*/;
    sparc_jump_table->a3_data[dill_jmp_divu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_divul] = sparc_div;
    sparc_jump_table->a3_data[dill_jmp_divul].data1 = 0x0d/*udiv*/;
    sparc_jump_table->a3_data[dill_jmp_divul].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_divi] = sparc_div;
    sparc_jump_table->a3_data[dill_jmp_divi].data1 = 0x2d/*sdivx*/;
    sparc_jump_table->a3_data[dill_jmp_divi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_divl] = sparc_div;
    sparc_jump_table->a3_data[dill_jmp_divl].data1 = 0x2d/*sdivx*/;
    sparc_jump_table->a3_data[dill_jmp_divl].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_modu] = sparc_mod;
    sparc_jump_table->a3_data[dill_jmp_modu].data1 = 0x0/*umod*/;
    sparc_jump_table->a3_data[dill_jmp_modu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_modul] = sparc_mod;
    sparc_jump_table->a3_data[dill_jmp_modul].data1 = 0x0/*umod*/;
    sparc_jump_table->a3_data[dill_jmp_modul].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_modi] = sparc_mod;
    sparc_jump_table->a3_data[dill_jmp_modi].data1 = 0x1/*smod*/;
    sparc_jump_table->a3_data[dill_jmp_modi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_modl] = sparc_mod;
    sparc_jump_table->a3_data[dill_jmp_modl].data1 = 0x1/*smod*/;
    sparc_jump_table->a3_data[dill_jmp_modl].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_andi] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_andi].data1 = 0x1;
    sparc_jump_table->a3_data[dill_jmp_andi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_andu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_andu].data1 = 0x1;
    sparc_jump_table->a3_data[dill_jmp_andu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_andul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_andul].data1 = 0x1;
    sparc_jump_table->a3_data[dill_jmp_andul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_andl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_andl].data1 = 0x1;
    sparc_jump_table->a3_data[dill_jmp_andl].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_ori] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_ori].data1 = 0x2;
    sparc_jump_table->a3_data[dill_jmp_ori].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_oru] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_oru].data1 = 0x2;
    sparc_jump_table->a3_data[dill_jmp_oru].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_orul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_orul].data1 = 0x2;
    sparc_jump_table->a3_data[dill_jmp_orul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_orl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_orl].data1 = 0x2;
    sparc_jump_table->a3_data[dill_jmp_orl].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_xori] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_xori].data1 = 0x3;
    sparc_jump_table->a3_data[dill_jmp_xori].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_xoru] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_xoru].data1 = 0x3;
    sparc_jump_table->a3_data[dill_jmp_xoru].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_xorul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_xorul].data1 = 0x3;
    sparc_jump_table->a3_data[dill_jmp_xorul].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_xorl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_xorl].data1 = 0x3;
    sparc_jump_table->a3_data[dill_jmp_xorl].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_lshi] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_lshi].data1 = 0x25;
    sparc_jump_table->a3_data[dill_jmp_lshi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_lshu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_lshu].data1 = 0x25;
    sparc_jump_table->a3_data[dill_jmp_lshu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_lshul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_lshul].data1 = 0x25;
    sparc_jump_table->a3_data[dill_jmp_lshul].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_lshl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_lshl].data1 = 0x25;
    sparc_jump_table->a3_data[dill_jmp_lshl].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_rshi] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_rshi].data1 = 0x27;
    sparc_jump_table->a3_data[dill_jmp_rshi].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_rshl] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_rshl].data1 = 0x27;
    sparc_jump_table->a3_data[dill_jmp_rshl].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_rshu] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_rshu].data1 = 0x26;
    sparc_jump_table->a3_data[dill_jmp_rshu].data2 = 0;
    sparc_jump_table->jmp_a3[dill_jmp_rshul] = sparc_FORM3_arith;
    sparc_jump_table->a3_data[dill_jmp_rshul].data1 = 0x26;
    sparc_jump_table->a3_data[dill_jmp_rshul].data2 = 1;
    sparc_jump_table->jmp_a3[dill_jmp_addf] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_addf].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_addf].data2 = 0x041;
    sparc_jump_table->jmp_a3[dill_jmp_addd] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_addd].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_addd].data2 = 0x042;
    sparc_jump_table->jmp_a3[dill_jmp_subf] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_subf].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_subf].data2 = 0x045;
    sparc_jump_table->jmp_a3[dill_jmp_subd] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_subd].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_subd].data2 = 0x046;
    sparc_jump_table->jmp_a3[dill_jmp_mulf] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_mulf].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_mulf].data2 = 0x049;
    sparc_jump_table->jmp_a3[dill_jmp_muld] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_muld].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_muld].data2 = 0x04a;
    sparc_jump_table->jmp_a3[dill_jmp_divf] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_divf].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_divf].data2 = 0x04d;
    sparc_jump_table->jmp_a3[dill_jmp_divd] = sparc_FORM3_farith;
    sparc_jump_table->a3_data[dill_jmp_divd].data1 = 0x34;
    sparc_jump_table->a3_data[dill_jmp_divd].data2 = 0x04e;
    sparc_jump_table->jmp_a3i[dill_jmp_addi] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_addi].data1 = 0x0;
    sparc_jump_table->a3i_data[dill_jmp_addi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_addu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_addu].data1 = 0x0;
    sparc_jump_table->a3i_data[dill_jmp_addu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_addul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_addul].data1 = 0x0;
    sparc_jump_table->a3i_data[dill_jmp_addul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_addl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_addl].data1 = 0x0;
    sparc_jump_table->a3i_data[dill_jmp_addl].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_addp] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_addp].data1 = 0x0;
    sparc_jump_table->a3i_data[dill_jmp_addp].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_subi] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_subi].data1 = 0x4;
    sparc_jump_table->a3i_data[dill_jmp_subi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_subu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_subu].data1 = 0x4;
    sparc_jump_table->a3i_data[dill_jmp_subu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_subul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_subul].data1 = 0x4;
    sparc_jump_table->a3i_data[dill_jmp_subul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_subl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_subl].data1 = 0x4;
    sparc_jump_table->a3i_data[dill_jmp_subl].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_subp] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_subp].data1 = 0x4;
    sparc_jump_table->a3i_data[dill_jmp_subp].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_mulu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_mulu].data1 = 0x9/*umul*/;
    sparc_jump_table->a3i_data[dill_jmp_mulu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_mulul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_mulul].data1 = 0x9/*umul*/;
    sparc_jump_table->a3i_data[dill_jmp_mulul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_muli] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_muli].data1 = 0x9/*smul*/;
    sparc_jump_table->a3i_data[dill_jmp_muli].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_mull] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_mull].data1 = 0x9/*smul*/;
    sparc_jump_table->a3i_data[dill_jmp_mull].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_divu] = sparc_divi;
    sparc_jump_table->a3i_data[dill_jmp_divu].data1 = 0x0d/*udiv*/;
    sparc_jump_table->a3i_data[dill_jmp_divu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_divul] = sparc_divi;
    sparc_jump_table->a3i_data[dill_jmp_divul].data1 = 0x0d/*udiv*/;
    sparc_jump_table->a3i_data[dill_jmp_divul].data2 = 1;
    sparc_jump_table->jmp_a3i[dill_jmp_divi] = sparc_divi;
    sparc_jump_table->a3i_data[dill_jmp_divi].data1 = 0x2d/*sdiv*/;
    sparc_jump_table->a3i_data[dill_jmp_divi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_divl] = sparc_divi;
    sparc_jump_table->a3i_data[dill_jmp_divl].data1 = 0x2d/*sdiv*/;
    sparc_jump_table->a3i_data[dill_jmp_divl].data2 = 1;
    sparc_jump_table->jmp_a3i[dill_jmp_modu] = sparc_modi;
    sparc_jump_table->a3i_data[dill_jmp_modu].data1 = 0x0/*umod*/;
    sparc_jump_table->a3i_data[dill_jmp_modu].data2 = 2;
    sparc_jump_table->jmp_a3i[dill_jmp_modul] = sparc_modi;
    sparc_jump_table->a3i_data[dill_jmp_modul].data1 = 0x0/*umod*/;
    sparc_jump_table->a3i_data[dill_jmp_modul].data2 = 2;
    sparc_jump_table->jmp_a3i[dill_jmp_modi] = sparc_modi;
    sparc_jump_table->a3i_data[dill_jmp_modi].data1 = 0x1/*smod*/;
    sparc_jump_table->a3i_data[dill_jmp_modi].data2 = 2;
    sparc_jump_table->jmp_a3i[dill_jmp_modl] = sparc_modi;
    sparc_jump_table->a3i_data[dill_jmp_modl].data1 = 0x1/*smod*/;
    sparc_jump_table->a3i_data[dill_jmp_modl].data2 = 2;
    sparc_jump_table->jmp_a3i[dill_jmp_andi] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_andi].data1 = 0x1;
    sparc_jump_table->a3i_data[dill_jmp_andi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_andu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_andu].data1 = 0x1;
    sparc_jump_table->a3i_data[dill_jmp_andu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_andul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_andul].data1 = 0x1;
    sparc_jump_table->a3i_data[dill_jmp_andul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_andl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_andl].data1 = 0x1;
    sparc_jump_table->a3i_data[dill_jmp_andl].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_ori] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_ori].data1 = 0x2;
    sparc_jump_table->a3i_data[dill_jmp_ori].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_oru] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_oru].data1 = 0x2;
    sparc_jump_table->a3i_data[dill_jmp_oru].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_orul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_orul].data1 = 0x2;
    sparc_jump_table->a3i_data[dill_jmp_orul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_orl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_orl].data1 = 0x2;
    sparc_jump_table->a3i_data[dill_jmp_orl].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_xori] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_xori].data1 = 0x3;
    sparc_jump_table->a3i_data[dill_jmp_xori].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_xoru] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_xoru].data1 = 0x3;
    sparc_jump_table->a3i_data[dill_jmp_xoru].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_xorul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_xorul].data1 = 0x3;
    sparc_jump_table->a3i_data[dill_jmp_xorul].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_xorl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_xorl].data1 = 0x3;
    sparc_jump_table->a3i_data[dill_jmp_xorl].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_lshi] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_lshi].data1 = 0x25;
    sparc_jump_table->a3i_data[dill_jmp_lshi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_lshu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_lshu].data1 = 0x25;
    sparc_jump_table->a3i_data[dill_jmp_lshu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_lshul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_lshul].data1 = 0x25;
    sparc_jump_table->a3i_data[dill_jmp_lshul].data2 = 1;
    sparc_jump_table->jmp_a3i[dill_jmp_lshl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_lshl].data1 = 0x25;
    sparc_jump_table->a3i_data[dill_jmp_lshl].data2 = 1;
    sparc_jump_table->jmp_a3i[dill_jmp_rshi] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_rshi].data1 = 0x27;
    sparc_jump_table->a3i_data[dill_jmp_rshi].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_rshl] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_rshl].data1 = 0x27;
    sparc_jump_table->a3i_data[dill_jmp_rshl].data2 = 1;
    sparc_jump_table->jmp_a3i[dill_jmp_rshu] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_rshu].data1 = 0x26;
    sparc_jump_table->a3i_data[dill_jmp_rshu].data2 = 0;
    sparc_jump_table->jmp_a3i[dill_jmp_rshul] = sparc_FORM3imm_arith;
    sparc_jump_table->a3i_data[dill_jmp_rshul].data1 = 0x26;
    sparc_jump_table->a3i_data[dill_jmp_rshul].data2 = 1;
    sparc_jump_table->jmp_a2[dill_jmp_comi] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_comi].data1 = 0x07;
    sparc_jump_table->a2_data[dill_jmp_comi].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_comu] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_comu].data1 = 0x07;
    sparc_jump_table->a2_data[dill_jmp_comu].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_comul] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_comul].data1 = 0x07;
    sparc_jump_table->a2_data[dill_jmp_comul].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_coml] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_coml].data1 = 0x07;
    sparc_jump_table->a2_data[dill_jmp_coml].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_negi] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_negi].data1 = 0x04;
    sparc_jump_table->a2_data[dill_jmp_negi].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_negu] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_negu].data1 = 0x04;
    sparc_jump_table->a2_data[dill_jmp_negu].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_negul] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_negul].data1 = 0x04;
    sparc_jump_table->a2_data[dill_jmp_negul].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_negl] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_negl].data1 = 0x04;
    sparc_jump_table->a2_data[dill_jmp_negl].data2 = 2;
    sparc_jump_table->jmp_a2[dill_jmp_noti] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_noti].data1 = dill_jmp_noti;
    sparc_jump_table->a2_data[dill_jmp_noti].data2 = 0;
    sparc_jump_table->jmp_a2[dill_jmp_notu] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_notu].data1 = dill_jmp_noti;
    sparc_jump_table->a2_data[dill_jmp_notu].data2 = 0;
    sparc_jump_table->jmp_a2[dill_jmp_notul] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_notul].data1 = dill_jmp_noti;
    sparc_jump_table->a2_data[dill_jmp_notul].data2 = 0;
    sparc_jump_table->jmp_a2[dill_jmp_notl] = (arith_op2)sparc_FORM2_arith;
    sparc_jump_table->a2_data[dill_jmp_notl].data1 = dill_jmp_noti;
    sparc_jump_table->a2_data[dill_jmp_notl].data2 = 0;
    sparc_jump_table->jmp_a2[dill_jmp_negf] = sparc_FORM2_farith;
    sparc_jump_table->a2_data[dill_jmp_negf].data1 = 0x34;
    sparc_jump_table->a2_data[dill_jmp_negf].data2 = 0x05;
    sparc_jump_table->jmp_a2[dill_jmp_negd] = sparc_FORM2_farith;
    sparc_jump_table->a2_data[dill_jmp_negd].data1 = 0x34;
    sparc_jump_table->a2_data[dill_jmp_negd].data2 = 0x06;
    sparc_jump_table->jmp_a2[dill_jmp_bswaps] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswaps].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswaps].data2 = DILL_S;
    sparc_jump_table->jmp_a2[dill_jmp_bswapus] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapus].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapus].data2 = DILL_US;
    sparc_jump_table->jmp_a2[dill_jmp_bswapi] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapi].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapi].data2 = DILL_I;
    sparc_jump_table->jmp_a2[dill_jmp_bswapu] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapu].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapu].data2 = DILL_U;
    sparc_jump_table->jmp_a2[dill_jmp_bswapul] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapul].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapul].data2 = DILL_UL;
    sparc_jump_table->jmp_a2[dill_jmp_bswapl] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapl].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapl].data2 = DILL_L;
    sparc_jump_table->jmp_a2[dill_jmp_bswapf] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapf].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapf].data2 = DILL_F;
    sparc_jump_table->jmp_a2[dill_jmp_bswapd] = sparc_bswap;
    sparc_jump_table->a2_data[dill_jmp_bswapd].data1 = 0;
    sparc_jump_table->a2_data[dill_jmp_bswapd].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_beqc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqc].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqc].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bequc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bequc].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_bequc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_beqs] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqs].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqs].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bequs] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bequs].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_bequs].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_beqi] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqi].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqi].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bequ] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bequ].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_bequ].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bequl] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bequl].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_bequl].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_beql] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beql].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beql].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_beqp] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqp].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqp].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_beqd] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqd].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqd].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_beqf] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_beqf].data1 = dill_eq_code;
    sparc_jump_table->b_data[dill_jmp_beqf].data2 = DILL_F;
    sparc_jump_table->jmp_b[dill_jmp_bgec] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgec].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgec].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bgeuc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgeuc].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgeuc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_bges] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bges].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bges].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bgeus] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgeus].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgeus].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_bgei] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgei].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgei].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bgeu] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgeu].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgeu].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bgeul] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgeul].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgeul].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_bgel] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgel].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgel].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_bgep] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgep].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgep].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_bged] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bged].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bged].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_bgef] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgef].data1 = dill_ge_code;
    sparc_jump_table->b_data[dill_jmp_bgef].data2 = DILL_F;
    sparc_jump_table->jmp_b[dill_jmp_bgtc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtc].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtc].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bgtuc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtuc].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtuc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_bgts] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgts].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgts].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bgtus] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtus].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtus].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_bgti] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgti].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgti].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bgtu] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtu].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtu].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bgtul] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtul].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtul].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_bgtl] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtl].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtl].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_bgtp] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtp].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtp].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_bgtd] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtd].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtd].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_bgtf] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bgtf].data1 = dill_gt_code;
    sparc_jump_table->b_data[dill_jmp_bgtf].data2 = DILL_F;
    sparc_jump_table->jmp_b[dill_jmp_blec] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blec].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_blec].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bleuc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bleuc].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bleuc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_bles] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bles].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bles].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bleus] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bleus].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bleus].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_blei] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blei].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_blei].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bleu] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bleu].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bleu].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bleul] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bleul].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bleul].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_blel] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blel].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_blel].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_blep] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blep].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_blep].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_bled] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bled].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_bled].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_blef] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blef].data1 = dill_le_code;
    sparc_jump_table->b_data[dill_jmp_blef].data2 = DILL_F;
    sparc_jump_table->jmp_b[dill_jmp_bltc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltc].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltc].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bltuc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltuc].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltuc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_blts] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blts].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_blts].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bltus] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltus].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltus].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_blti] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_blti].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_blti].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bltu] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltu].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltu].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bltul] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltul].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltul].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_bltl] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltl].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltl].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_bltp] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltp].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltp].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_bltd] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltd].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltd].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_bltf] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bltf].data1 = dill_lt_code;
    sparc_jump_table->b_data[dill_jmp_bltf].data2 = DILL_F;
    sparc_jump_table->jmp_b[dill_jmp_bnec] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnec].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnec].data2 = DILL_C;
    sparc_jump_table->jmp_b[dill_jmp_bneuc] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bneuc].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bneuc].data2 = DILL_UC;
    sparc_jump_table->jmp_b[dill_jmp_bnes] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnes].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnes].data2 = DILL_S;
    sparc_jump_table->jmp_b[dill_jmp_bneus] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bneus].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bneus].data2 = DILL_US;
    sparc_jump_table->jmp_b[dill_jmp_bnei] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnei].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnei].data2 = DILL_I;
    sparc_jump_table->jmp_b[dill_jmp_bneu] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bneu].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bneu].data2 = DILL_U;
    sparc_jump_table->jmp_b[dill_jmp_bneul] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bneul].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bneul].data2 = DILL_UL;
    sparc_jump_table->jmp_b[dill_jmp_bnel] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnel].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnel].data2 = DILL_L;
    sparc_jump_table->jmp_b[dill_jmp_bnep] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnep].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnep].data2 = DILL_P;
    sparc_jump_table->jmp_b[dill_jmp_bned] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bned].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bned].data2 = DILL_D;
    sparc_jump_table->jmp_b[dill_jmp_bnef] = sparc_branch;
    sparc_jump_table->b_data[dill_jmp_bnef].data1 = dill_ne_code;
    sparc_jump_table->b_data[dill_jmp_bnef].data2 = DILL_F;
    sparc_jump_table->jmp_bi[dill_jmp_beqc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bequc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_beqs] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bequs] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_beqi] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bequ] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bequl] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_beql] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_beqp] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgec] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgeuc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bges] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgeus] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgei] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgeu] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgeul] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgel] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgep] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtuc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgts] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtus] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgti] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtu] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtul] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtl] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bgtp] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blec] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bleuc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bles] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bleus] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blei] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bleu] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bleul] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blel] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blep] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltuc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blts] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltus] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_blti] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltu] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltul] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltl] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bltp] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bnec] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bneuc] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bnes] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bneus] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bnei] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bneu] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bneul] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bnel] = sparc_branchi;
    sparc_jump_table->jmp_bi[dill_jmp_bnep] = sparc_branchi;
  sparc_jump_table->jmp_c[dill_jmp_ceqc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqc].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqc].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cequc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cequc].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_cequc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_ceqs] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqs].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqs].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cequs] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cequs].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_cequs].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_ceqi] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqi].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqi].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cequ] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cequ].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_cequ].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cequl] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cequl].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_cequl].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_ceql] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceql].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceql].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_ceqp] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqp].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqp].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_ceqd] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqd].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqd].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_ceqf] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_ceqf].data1 = dill_eq_code;
  sparc_jump_table->c_data[dill_jmp_ceqf].data2 = DILL_F;
  sparc_jump_table->jmp_c[dill_jmp_cgec] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgec].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgec].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cgeuc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgeuc].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgeuc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_cges] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cges].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cges].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cgeus] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgeus].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgeus].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_cgei] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgei].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgei].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cgeu] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgeu].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgeu].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cgeul] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgeul].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgeul].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_cgel] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgel].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgel].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_cgep] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgep].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgep].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_cged] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cged].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cged].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_cgef] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgef].data1 = dill_ge_code;
  sparc_jump_table->c_data[dill_jmp_cgef].data2 = DILL_F;
  sparc_jump_table->jmp_c[dill_jmp_cgtc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtc].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtc].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cgtuc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtuc].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtuc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_cgts] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgts].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgts].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cgtus] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtus].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtus].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_cgti] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgti].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgti].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cgtu] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtu].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtu].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cgtul] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtul].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtul].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_cgtl] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtl].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtl].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_cgtp] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtp].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtp].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_cgtd] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtd].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtd].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_cgtf] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cgtf].data1 = dill_gt_code;
  sparc_jump_table->c_data[dill_jmp_cgtf].data2 = DILL_F;
  sparc_jump_table->jmp_c[dill_jmp_clec] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clec].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_clec].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cleuc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cleuc].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cleuc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_cles] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cles].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cles].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cleus] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cleus].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cleus].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_clei] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clei].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_clei].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cleu] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cleu].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cleu].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cleul] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cleul].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cleul].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_clel] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clel].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_clel].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_clep] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clep].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_clep].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_cled] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cled].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_cled].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_clef] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clef].data1 = dill_le_code;
  sparc_jump_table->c_data[dill_jmp_clef].data2 = DILL_F;
  sparc_jump_table->jmp_c[dill_jmp_cltc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltc].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltc].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cltuc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltuc].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltuc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_clts] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clts].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_clts].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cltus] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltus].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltus].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_clti] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_clti].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_clti].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cltu] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltu].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltu].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cltul] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltul].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltul].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_cltl] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltl].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltl].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_cltp] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltp].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltp].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_cltd] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltd].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltd].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_cltf] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cltf].data1 = dill_lt_code;
  sparc_jump_table->c_data[dill_jmp_cltf].data2 = DILL_F;
  sparc_jump_table->jmp_c[dill_jmp_cnec] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnec].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnec].data2 = DILL_C;
  sparc_jump_table->jmp_c[dill_jmp_cneuc] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cneuc].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cneuc].data2 = DILL_UC;
  sparc_jump_table->jmp_c[dill_jmp_cnes] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnes].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnes].data2 = DILL_S;
  sparc_jump_table->jmp_c[dill_jmp_cneus] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cneus].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cneus].data2 = DILL_US;
  sparc_jump_table->jmp_c[dill_jmp_cnei] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnei].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnei].data2 = DILL_I;
  sparc_jump_table->jmp_c[dill_jmp_cneu] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cneu].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cneu].data2 = DILL_U;
  sparc_jump_table->jmp_c[dill_jmp_cneul] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cneul].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cneul].data2 = DILL_UL;
  sparc_jump_table->jmp_c[dill_jmp_cnel] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnel].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnel].data2 = DILL_L;
  sparc_jump_table->jmp_c[dill_jmp_cnep] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnep].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnep].data2 = DILL_P;
  sparc_jump_table->jmp_c[dill_jmp_cned] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cned].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cned].data2 = DILL_D;
  sparc_jump_table->jmp_c[dill_jmp_cnef] = sparc_compare;
  sparc_jump_table->c_data[dill_jmp_cnef].data1 = dill_ne_code;
  sparc_jump_table->c_data[dill_jmp_cnef].data2 = DILL_F;
  sparc_jump_table->jmp_ci[dill_jmp_ceqc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cequc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_ceqs] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cequs] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_ceqi] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cequ] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cequl] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_ceql] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_ceqp] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgec] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgeuc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cges] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgeus] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgei] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgeu] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgeul] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgel] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgep] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtuc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgts] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtus] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgti] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtu] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtul] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtl] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cgtp] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clec] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cleuc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cles] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cleus] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clei] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cleu] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cleul] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clel] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clep] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltuc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clts] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltus] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_clti] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltu] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltul] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltl] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cltp] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cnec] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cneuc] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cnes] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cneus] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cnei] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cneu] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cneul] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cnel] = sparc_comparei;
  sparc_jump_table->jmp_ci[dill_jmp_cnep] = sparc_comparei;
    sparc_jump_table->proc_start = (dill_mach_proc_start) sparc_proc_start;
    sparc_jump_table->end = sparc_end;
    sparc_jump_table->package_end = sparc_package_end;
    sparc_jump_table->clone_code = sparc_clone_code;
    sparc_jump_table->type_align = sparc_type_align;
    sparc_jump_table->type_size = sparc_type_size;
    sparc_jump_table->ret = sparc_ret;
    sparc_jump_table->reti = sparc_reti;
    sparc_jump_table->retf = (ret_opf)sparc_reti;
    sparc_jump_table->load = sparc_pload;
    sparc_jump_table->bsload = sparc_pbsload;
    sparc_jump_table->loadi = sparc_ploadi;
    sparc_jump_table->bsloadi = sparc_pbsloadi;
    sparc_jump_table->loadi = sparc_ploadi;
    sparc_jump_table->store = sparc_pstore;
    sparc_jump_table->storei = sparc_pstorei;
    sparc_jump_table->convert = sparc_convert;
    sparc_jump_table->mov = sparc_mov;
    sparc_jump_table->set = sparc_pset;
    sparc_jump_table->setf = sparc_setf;
    sparc_jump_table->setp = sparc_setp;
    sparc_jump_table->jv = sparc_jump_to_label;
    sparc_jump_table->jp = sparc_jump_to_reg;
    sparc_jump_table->jpi = sparc_jump_to_imm;
    sparc_jump_table->jal = sparc_jal;
    sparc_jump_table->push = sparc_push;
    sparc_jump_table->pushi = sparc_pushi;
    sparc_jump_table->pushfi = sparc_pushfi;
    sparc_jump_table->pushpi = sparc_pushpi;
    sparc_jump_table->calli = sparc_calli;
    sparc_jump_table->callr = sparc_callr;
    sparc_jump_table->local = sparc_local_op;
    sparc_jump_table->save_restore = sparc_save_restore_op;
    sparc_jump_table->init_disassembly = sparc_init_disassembly_info;
    sparc_jump_table->print_insn = sparc_print_insn;
    sparc_jump_table->print_reg = sparc_print_reg;
    sparc_jump_table->count_insn = sparc_count_insn;
    sparc_jump_table->do_reverse_push = 0;
    sparc_jump_table->target_byte_order = 1;  /* Format_Integer_bigendian */
    sparc_jump_table->target_float_format = 1;  /* Format_IEEE_754_bigendian */ 
}

extern void dill_sparc_init(dill_stream s)
{
    gen_sparc_jump_table();
    s->j = sparc_jump_table;
    s->dill_local_pointer = _fp;
    s->dill_param_reg_pointer = _fp;
    s->p->mach_info = (void*)gen_sparc_mach_info(s,/*v9*/ 0);
    s->p->machine_strr_tmp_reg = _g1;
    return;
}
extern void dill_sparcv9_init(dill_stream s)
{
        gen_sparc_jump_table();
    s->j = sparc_jump_table;
    s->dill_local_pointer = _fp;
    s->dill_param_reg_pointer = _fp;
    s->p->mach_info = (void*)gen_sparc_mach_info(s,/*v9*/ 1);
    s->p->machine_strr_tmp_reg = _g1;
    return;
}
