/*
 * SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef ADIOS2_HELPER_ADIOSMEMORY_TCC_
#define ADIOS2_HELPER_ADIOSMEMORY_TCC_

#include "adiosMemory.h"

#include "adios2/common/ADIOSMacros.h"

namespace adios2
{
namespace helper
{} // end namespace helper
} // end namespace adios2

#endif /* ADIOS2_HELPER_ADIOSMEMORY_TCC_ */
